package it.polito.elite.doseWS.agency;

import java.util.*;
import javax.xml.namespace.QName;
import javax.xml.rpc.Call;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceFactory;
import jade.content.lang.sl.*;
import jade.core.*;
import jade.lang.acl.ACLMessage;
import jade.domain.mobility.*;
import it.polito.elite.doseWS.agency.behaviours.*;
import it.polito.elite.doseWS.agency.messages.*;
import it.polito.elite.doseWS.agency.parsers.*;
import jade.wrapper.*;
import jade.content.*;
import jade.content.onto.basic.*;
import jade.domain.JADEAgentManagement.*;
import jade.domain.FIPAAgentManagement.*;
import jade.lang.acl.*;
import jade.domain.*;
import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 * <p>Title: AgencyManager</p>
 * <p>Description: The agent delegated to manage the whole agency. Listens from WS
 * "object world" requests via a ListenObjectsBehaviour. It starts up the WS and
 * the remote containers. It creates the rest of the agents, kills not respondings
 * and instances new ones. Monitors the knowledge base sunken in the DB manged
 * by the AnnotationRepository WS.  </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author Alessio Bosca
 */

public class AgencyManager extends DOSEAgent
{
  //define the service identifier
  public static final String SERVICE = "AgencyManagement";
  //wait for Containers startup
  private static final long WaitForContainersSetup = 10000;
  //Is Agency ready?
  private boolean isReady = false;
  //Web Services EndPoints
  private String annotationEP, expanderEP, searchEP;
  //endpoint as keys, values = locations
  private Hashtable AvailableContainers; //String -> Location
  //Central agency HostName
  private String thisHost;
  //DB stuff
  private String dbURI, dbUSR, dbPWD, dbDRV;
  //Stopword & Ontology File
  private String StopWordURI;
  private String ontoURI;
  //MessageParser
  private MsgParser msgParser;
  //HashSets containing available agent AIDs
  public HashSet Filters, Annotators, SynsetManagers;
  public Hashtable PendingEmployees;
  //Log PrintStream
  private PrintStream logBW;
  static private String logFileName="DoseAgency.log";

  /**
   * Manages the startup phase of the agent by registering to the services of
   * plattform, collecting the initialization params, creating the agency employees
   * and starting the management behaviours
   */
  public void setup()
  {
    //Enabling & Registering
    this.setEnabledO2ACommunication(true, 10);
    this.getContentManager().registerLanguage(new SLCodec());
    this.getContentManager().registerOntology(MobilityOntology.getInstance());
    this.register(AgencyManager.SERVICE);
    //Message Parser
    this.msgParser = new MsgParser();
    this.msgParser.register(ImAlive.class);
    this.msgParser.register(IndexReportMessage.class);
    this.msgParser.register(RemoteLoggingMessage.class);
    //Employee Collectors setup and Registration message subscription
    this.Filters = new HashSet();
    this.Annotators = new HashSet();
    this.SynsetManagers = new HashSet();
    this.subscribeServiceRegistrations(FilterAgent.SERVICE);
    this.subscribeServiceRegistrations(AnnotationAgent.SERVICE);
    this.subscribeServiceRegistrations(SynsetManager.SERVICE);
    //LogFile
    try {
      this.logBW = new PrintStream(new FileOutputStream(AgencyManager.logFileName,true));
      this.logBW.println("------------------NEW SESSION---------------------");
    }
    catch (IOException ex) {
      //failed to create file, log redirected to stdout (Web Application Server Logs..)
      logBW = System.out;
    }

    //getting setup parameters
    Object args[] = getArguments();
    //globals
    this.thisHost = (String) args[0];
    this.annotationEP = (String) args[1];
    this.expanderEP = (String) args[2];
    this.searchEP = (String) args[3];
    Integer delayMinutes = (Integer) args[4];
    //Data Sources
    this.StopWordURI = (String) args[5];
    this.ontoURI = (String) args[6];
    this.dbURI = (String) args[7];
    this.dbUSR = (String) args[8];
    this.dbPWD = (String) args[9];
    this.dbDRV = (String) args[10];
    //Containers
    HashSet ContainerProvidersEP = null;
    ContainerProvidersEP = (HashSet) args[11];
    this.log(this.getLocalName() + ": I'm Alive");

    //distributed containers initializing
    this.setupAvailableContainers(ContainerProvidersEP);
    this.doWait(AgencyManager.WaitForContainersSetup);
    this.getAvailableContainers();

    //Agents creation
    this.createSynsetManager();
    java.util.Random r = new java.util.Random(System.currentTimeMillis());
    int id;
    Enumeration e = this.AvailableContainers.keys();
    while (e.hasMoreElements()) {
      Location loc = (Location)this.AvailableContainers.get(e.nextElement());
      this.logBW.println(">>>Found container: "+loc);
      id = r.nextInt(1000);
      this.createAnnotationAgent(id, loc, this.annotationEP);
      this.createFilterAgent(id, loc);
    }

    //Starting communication & management Behaviours
    this.addBehaviour(new ListenObjectBehaviour(this));
    this.addBehaviour(new ListenMessageBehaviour(this));
  }

  /**
   * Overrides the same method of DOSEAgent. Handles an incoming ACLMessage
   * from the agency
   * @param msg Incoming Message
   */
  public void handleMessage(ACLMessage msg)
  {
    MessageTemplate mt = MessageTemplate.MatchSender(getDefaultDF());
    //If msg sender it is DF, it's a subscribtion msg
    if (mt.match(msg)) {
      //update Employee collectors adding new registering service provider agents
      this.handleDFMessage(msg);
    }
    else {
      //Decode the message
      Message decodedMsg = msgParser.routeMessage(msg.getContent());
      //Swithc between supported messages
      if (decodedMsg instanceof ImAlive) {
	//Remove from the PendingEmployees hashtable..
	ImAlive resp = (ImAlive) decodedMsg;

      }
      else if (decodedMsg instanceof IndexReportMessage) {
	//Remove from the PendingRequests HashTable....
	IndexReportMessage idxrpt = (IndexReportMessage) decodedMsg;
	this.log("Completed " + idxrpt.getCompletedURL());
      }
      else if (decodedMsg instanceof RemoteLoggingMessage) {
	//Actually stores the Log message
	RemoteLoggingMessage rl = (RemoteLoggingMessage) decodedMsg;
	this.log(rl.getAgent() + " from " + rl.getLocation() + " said \"" +
		 rl.getLog() + "\"");
      }
      else {
	this.log("UnHandledMsg >>\n" + msg.getContent());
      }
    }
  }

  /**
   * Handles an incoming ACLMessage from DF reporting the registration of an agent
   * to a Service of a certain type.
   * @param msg Incoming Message
   */
  private void handleDFMessage(ACLMessage msg)
  {
    try {
      //Get the agent descriptions from the df
      DFAgentDescription[] dfds = DFService.decodeNotification(msg.getContent());
      //iterates over the descriptions
      for (int i = 0; i < dfds.length; i++) {
	jade.util.leap.Iterator it = dfds[i].getAllServices();
	ServiceDescription dsc = (ServiceDescription) it.next();
	//get the string describing the service from agent descriptor
	String str = (String) dsc.getType();

	//Register agent AID in the apposite collector
	if (str.equals(FilterAgent.SERVICE)) {
	  Filters.add(dfds[i].getName());
	}
	else if (str.equals(AnnotationAgent.SERVICE)) {
	  Annotators.add(dfds[i].getName());
	}
	else if (str.equals(SynsetManager.SERVICE)) {
	  SynsetManagers.add(dfds[i].getName());
	}
	//When at least one agent of each kind has started set the Agency to ready
	if (!this.isReady) {
	  if ( (Filters.size() > 0) && (Annotators.size() > 0) &&
	      (SynsetManagers.size() > 0)) {
	    this.isReady = true;
	    this.log("Agency ready to go!!");
	  }
	}

      } //end for
    }
    catch (Exception ex) {
      this.log("!! HandleDFMessage: "+ex.toString());
    }
  }

  /**
   * Subscribe to DF in order to receive msgs whenever a new agents registers
   * for a service with the specified description
   * @param service Description of the service
   */
  private void subscribeServiceRegistrations(String service)
  {
    DFAgentDescription template = new DFAgentDescription();
    ServiceDescription sd = new ServiceDescription();
    sd.setType(service);
    template.addServices(sd);
    send(DFService.createSubscriptionMessage(this, getDefaultDF(), template, null));
  }

  /**
   * Initialize Container Providers
   * @param Providers Collection of ContainerProvider EndPoints
   * @return the number of container succesfully initialized
   */
  private int setupAvailableContainers(HashSet Providers)
  {
    int cont = 0;
    Iterator it = Providers.iterator();
    while (it.hasNext()) {
      String endpoint = (String) it.next();
      if (initRemoteContainer(endpoint)) {
	cont++;
      }
    }
    //if not all the containers could be initialized prints a log message
    if (cont < Providers.size()) {
      this.log("Only " + cont + " of " + Providers.size() +
	       " total Containers started.");
    }
    return cont;
  }

  /**
   * Invoke the creation of a remote container through a proper WS
   * @param endpoint EndPoint of a ContainerProviderWS
   * @return true on success, false otherwise
   */
  private boolean initRemoteContainer(String endpoint)
  {
    Boolean ret;
    //Constructs the name of the container as the name of the machine "mymachine".polito.it
    int begin = endpoint.indexOf("://") + 3;
    int end = endpoint.indexOf('.');
    String name = endpoint.substring(begin, end);
    //Constructs the name of the server e.g. http://myserver.mynet.com
    end = endpoint.indexOf('/', begin);
    String server = endpoint.substring(0, end + 1);

    try {
      //creates a new QName identifier for the container providing service
      QName op = new QName(server, "createContainer");
      //creating service factory
      ServiceFactory factory = ServiceFactory.newInstance();
      //defining the service
      Service service = factory.createService(new QName(
	  "ContainerProviderWSService"));
      //defining the service port
      QName port = new QName("ContainerProviderWS");
      //creating the call object
      Call call = service.createCall(port);
      //sets the endpoint for the call
      call.setTargetEndpointAddress(endpoint);
      //sets the operation to be required by the call
      call.setOperationName(op);
      //invoking init operation
      ret = (Boolean) call.invoke(new Object[] {thisHost, name});
    }
    catch (Exception ex) {
      log("!! InitRemoteContainer: "+ex.toString());
      return false;
    }
    return ret.booleanValue();
  }

  /**
   * Get container locations by submitting a query to the AMS.
   * Such locations are stored in AvailableContainers Hashtable
   * @return true on success, false if an error has occour in the process
   */
  private boolean getAvailableContainers()
  {
    // Requests available container locations to the AMS
    ACLMessage request = new ACLMessage(ACLMessage.REQUEST);
    request.setLanguage(new SLCodec().getName());
    request.setOntology(MobilityOntology.getInstance().getName());

    try {
      Action action = new Action(this.getAMS(),
				 new QueryPlatformLocationsAction());
      this.getContentManager().fillContent(request, action);
      request.addReceiver(action.getActor());
      //Sends the request
      this.send(request);

      //Receive response from AMS
      MessageTemplate mt = MessageTemplate.and(
      MessageTemplate.MatchSender(this.getAMS()),
      MessageTemplate.MatchPerformative(ACLMessage.INFORM));
      ACLMessage resp = this.blockingReceive(mt);
      //Extracting content
      ContentElement ce = this.getContentManager().extractContent(resp);
      Result result = (Result) ce;
      jade.util.leap.Iterator it = result.getItems().iterator();

      //availableContainers collection clean up or instance
      if (this.AvailableContainers != null) {
	this.AvailableContainers.clear();
      }
      else {
	this.AvailableContainers = new Hashtable();
      }

      //new availableContainers collection built up
      while (it.hasNext()) {
	Location loc = (Location) it.next();
	if (!loc.getName().equals("Main-Container")) {
	  this.AvailableContainers.put(loc.getName(), loc);
	}
      }
    }
    catch (Exception ex) {
      log("!! getAvailableContainers: "+ex.toString());
      return false;
    }
    return true;
  }

  /**
   * Issues the indexing of a document, given its URL, to the agency
   * @param URI document URL
   * @return true on success, false otherwise
   */
  public boolean indexURI(String URI)
  {
    if (!this.isReady) {
      return false;
    }
    try {
      //Compose an indexing request message
      IndexRequestMessage reqq = new IndexRequestMessage(URI);
      ACLMessage msg = new ACLMessage(ACLMessage.INFORM);

      //Picks up a random filter agent in the agency
      java.util.Random r = new java.util.Random(System.currentTimeMillis());
      AID[] aids = new AID[1];
      aids = (AID[]) Filters.toArray(aids);
      int k = r.nextInt(aids.length);
      msg.addReceiver(aids[k]);

      //writes the actual message and sends it
      msg.setContent(reqq.createMsg());
      this.send(msg);
      return true;
    }
    catch (Exception ex) {
      this.log("!! indexURI: "+ex.toString());
      return false;
    }
  }

  private boolean createFilterAgent(int id, Location loc)
  {
    jade.wrapper.Agent wra;
    jade.wrapper.AgentContainer myContainer = getContainerController();

    //create the filter agent
    String name = "filter_" + id;
    Object args[] = new Object[3];
    args[0] = loc;
    args[1] = this.StopWordURI;
    args[2] = this.getAID();
    try {
      wra = (jade.wrapper.Agent) myContainer.createNewAgent(name,
	  "it.polito.elite.doseWS.agency.FilterAgent", args);
      wra.start();
    }
    catch (StaleProxyException ex) {
      this.log("!! Failed creating " + name + " agent\n" + ex.toString());
      return false;
    }
    return true;
  }

  private boolean createAnnotationAgent(int id, Location loc,
					String endpoint)
  {
    jade.wrapper.Agent wra;
    jade.wrapper.AgentContainer myContainer = getContainerController();

    //create the annotations agent
    String name = "annotator_" + id;
    Object args[] = new Object[3];
    args[0] = loc;
    args[1] = this.annotationEP;
    args[2] = this.getAID();

    try {
      wra = (jade.wrapper.Agent) myContainer.createNewAgent(name,
	  "it.polito.elite.doseWS.agency.AnnotationAgentXtfidf", args);
      wra.start();
    }
    catch (StaleProxyException ex) {
      this.log("!! Failed creating " + name + " agent\n" + ex.toString());
      return false;
    }
    return true;
  }

  private boolean createSynsetManager()
  {
    jade.wrapper.Agent wra;
    jade.wrapper.AgentContainer myContainer = getContainerController();

    //create the Synset Manager agent
    String name = "SynsetManager";
    Object args[] = new Object[6];
    args[0] = this.dbDRV;
    args[1] = this.dbUSR;
    args[2] = this.dbPWD;
    args[3] = this.dbURI;
    args[4] = this.ontoURI;
    args[5] = this.getAID();
    try {
      wra = (jade.wrapper.Agent) myContainer.createNewAgent(name,
	  "it.polito.elite.doseWS.agency.SynsetManager", args);
      wra.start();
    }
    catch (StaleProxyException ex) {
      this.log("!! Failed creating " + name + " agent\n" + ex.toString());
      return false;
    }
    return true;

  }

  /**
   * Logs an event
   * @param description String that describes the event
   */
  public void log(String description)
  {
    Date d = new Date(System.currentTimeMillis());
    SimpleDateFormat df = (SimpleDateFormat)DateFormat.getDateTimeInstance();
	df.applyPattern("yyyy:MM:dd:hh:mm:ss:S:Z");
    String ts = df.format(d);
    this.logBW.println("-"+ts+" "+description);
    this.logBW.flush();
  }

  /**
   * Tells the Annotation WS endpoint for this platform
   * @return A String representing the Annotation WS EndPoint
   */
  public String getAnnotationEP()
  {
    return this.annotationEP;
  }
  /**
   * Deregister from services and file closing
   */
  protected void takeDown()
  {
    try {
      DFService.deregister(this);
      this.logBW.close();
    }
    catch (Exception e) {}
  }


}
