package it.polito.elite.doseWS.agency;


import java.io.*;
import java.util.*;
import it.polito.elite.doseWS.include.*;
import it.polito.elite.doseWS.wrappers.SynsetWrapper;
import it.polito.elite.doseWS.wrappers.SynsetWrapperImpl;
import it.polito.elite.doseWS.agency.parsers.*;
import it.polito.elite.doseWS.agency.messages.*;
import it.polito.elite.doseWS.agency.behaviours.*;
import jade.core.*;
import jade.lang.acl.*;
import javax.xml.namespace.QName;
import javax.xml.rpc.Call;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceFactory;

/**
 *  * @author Bonino Dario, Alessio Bosca
 */


public class AnnotationAgentXtfidf extends DOSEAgent
{
	//the serial version identifier for granting compatibility between
	//agents on different platforms
	private static final long serialVersionUID = 1;
	
	//the service name
	static public final String SERVICE = "FragmentAnnotation";
	
	//the hash of mwords
	private HashSet Mwords;
	
	//the synsetWrapper for parsing synset data
	private SynsetWrapper synWrapper;
	
	//the stemmer
	private Stemmer stemmer;
	
	//the current language
	private String lang;
	
	//the ws endpoints for the annotation storage service
	public String serverAnnoWS, endpointAnnoWS;
	
	//the set of synset managers, and the set of received requests
	private HashSet synsetManager, requests;
	
	//the unique identifier of the manager agent
	private AID manager;
	
	//the msg parser for message routing
	private MsgParser msgParser;
	
	//the hashtable for already indexed tf data, one hash per language
	private Hashtable docHash;
	
	//a busy indicator
	protected boolean busy;
	
	//the number of documents currently indexed
	private int nDocs;
	
	//the expiration date of synsets (not required)
	private Date expirationDate;

  /**
   * Agent LifeCycle startup
   */
  public void setup()
  {
    //get initialization parameters
    Object params[] = this.getArguments();
    
    //the agent location
    Location loc = (Location) params[0];
    
    //the annotation storage service endpoint
    this.endpointAnnoWS = (String) params[1];
    
    //the manager agent
    this.manager = (AID) params[2];
    
    //get AnnoWS server
    int indx = this.endpointAnnoWS.indexOf("://") + 3;
    indx = this.endpointAnnoWS.indexOf("/", indx);
    this.serverAnnoWS = this.endpointAnnoWS.substring(0, indx + 1);
    
    //We have no synsets loaded on start-up, since the first request, so lang =""
    this.lang = new String("");
    
    //initialize the document hash
    this.docHash = new Hashtable();
    
    //sets the initial number of indexed documents to 0
    this.nDocs = 0;
    
    //moves to the final destination
    if (loc != null) {
      this.doMove(loc);
    }
    else {
      this.sendRemoteLog("!!Location of the destination container is null",manager);
      //Explicitly call afterMove when not migration the agent for debug pourposes
      //this.afterMove()
    }
  }

  /**
   * This method is invoked each time an agent is migrated to a new Location
   * and completes the agent setup phase.
   */
  public void afterMove()
  {
    //finishing setup...
  	
  	//creating the message parser for message routing
    this.msgParser = new MsgParser();
    
    //creating the requests hashset
    this.requests = new HashSet();
    
    //creating the hashset of available synset managers
    this.synsetManager = new HashSet();
    
    //creating a new stemmer
    this.stemmer = new Stemmer();
    
    //creating the hashtable of distinct mwords contained in the synsets
    this.Mwords = new HashSet();
    
    //sets the expiration date to now
    this.expirationDate = new Date();
    
    //msgs from Agency Manager
    this.msgParser.register(IsAlive.class);
    this.msgParser.register(Die.class);
    //msgs from Filter Agent
    this.msgParser.register(AnnotationPreRequestMessage.class);
    this.msgParser.register(AnnotationRequestMessage.class);
    this.msgParser.register(SynsetResponseMessage.class);
    
    //behaviour
    this.addBehaviour(new ListenMessageBehaviour(this));
    this.addBehaviour(new GetAgentProviders(this, 600000, SynsetManager.SERVICE,
					    synsetManager));
    
    //register to Directory Facilitator
    this.register(AnnotationAgentXtfidf.SERVICE);
  }

  /**
   * Overrides the same method of DOSEAgent. Handles an incoming ACLMessage
   * from the agency
   * @param msg Incoming Message
   */
  public void handleMessage(ACLMessage msg)
  {
  	//the message decoded as a specific type of registered messages
    Message decodedMsg = msgParser.routeMessage(msg.getContent());
    
    // IS-ALIVE
    if (decodedMsg instanceof IsAlive) {
      ImAlive content = new ImAlive(AnnotationAgentXtfidf.SERVICE);
      ACLMessage reply = msg.createReply();
      reply.setContent(content.createMsg());
      this.send(reply);
    }
    
    // ANNOTATION-PRE-REQUEST
    else if (decodedMsg instanceof AnnotationPreRequestMessage) {
      AnnotationPreRequestMessage preReq = (AnnotationPreRequestMessage)
	  decodedMsg;
      if (lang.equals(preReq.getLanguage())) {
	ACLMessage reply = msg.createReply();
	AnnotationOfferMessage resp = new AnnotationOfferMessage(this.expirationDate,
	    this.getCurQueueSize() + 1);
	reply.setContent(resp.createMsg());
	this.send(reply);
	this.sendRemoteLog("Annotation Offer: queue=" + resp.getRunnings() +
			   " exp=" + resp.getDate().toString(), this.manager);
      }
    }
    
    // ANNOTATION-REQUEST
    else if (decodedMsg instanceof AnnotationRequestMessage) 
    {
    	//extracts the decoded message
    	AnnotationRequestMessage req = (AnnotationRequestMessage) decodedMsg;
    	
    	//gets the language
    	String reqLang = req.getLanguage();
    	
    	//logs
    	this.sendRemoteLog("Received request of " + req.getURL(), manager);
    	
    	//if the current language is diverse from the document one 
    	if (!lang.equals(reqLang)) //|| (!this.isExpirationValid())) 
    	{
    		//Preparing the request of new Synsets
    		SynsetRequestMessage synReq = new SynsetRequestMessage(reqLang);
    		
    		//creating the corresponding ACL message
    		ACLMessage synReqMsg = new ACLMessage(ACLMessage.REQUEST);
    		
    		//setting the message content
    		synReqMsg.setContent(synReq.createMsg());
    		
    		//gets the list of synset managers available
    		Iterator it = this.synsetManager.iterator();
    		
    		//extracts the agent unique identifier for the first agent in the list
    		AID sm = (AID) it.next();
    		
    		//add the agent as receiver for the message
    		synReqMsg.addReceiver(sm);
    		
    		//sends the message
    		this.send(synReqMsg);
    		
    		//Receive the synset in a blocking way
    		MessageTemplate mt = MessageTemplate.MatchSender(sm);
    		ACLMessage synsMsg = this.blockingReceive(mt); //gestione timeout ed errore..
    		
    		//message received, sending remote log
    		this.sendRemoteLog("Received synsets... in " + reqLang, manager);
    		
    		//build the Synset Response Message
    		SynsetResponseMessage srm = (SynsetResponseMessage)SynsetResponseMessage.parse(synsMsg.getContent());
    		
    		//Load Synset into memory
    		if (!this.acquireSynsets(srm.getContent())) 
    		{
    			this.sendRemoteLog(
    					"!!! Failure on loading Synsets.. Discarding document " +
						req.getURL(), this.manager);
    			return;
    		}
    		
    		// as first initializes the stemmer
    	  	this.stemmer.initStemmer(Lang.getExtended(this.lang));
    	}
    	this.annotate(req.getText(), req.getURL());
    }
    
    // SYNSET-RESPONSE
    else if (decodedMsg instanceof SynsetResponseMessage) {
      // The message can only be received after an explicit Synset request
      // following a annotation request. If others arrives they are just discarded
      this.sendRemoteLog("!!Received unexpected not requested synsets",manager);
    }
    
    // DIE
    else if (decodedMsg instanceof Die) {//Explicit Die request
      this.doDelete();
    }

  }

  /**
   * Create the annotations and send them to AnnotationRepositoryWS. It further
   * compose a report message for the SynsetManager and another one for the agency
   * manager.
   * @param doc The text fragment to annotate
   * @param docURL the URl of the document to be annotated
   * @return true on success, false otherwise
   */
  public boolean annotate(String doc, String docURL)
  {
  	
  	//remote log
  	//this.sendRemoteLog("evaluating the tfidf values",this.manager);
  	
  	//the activation hashtable for each mword
  	Hashtable tfIdf = this.evaluateTfIdf(doc,docURL);
  	
  	//remote log
  	//this.sendRemoteLog("evaluating the activation values",this.manager);
  	
  	//evaluating the activation of each ontology concept
  	Hashtable activation = this.evaluateConceptActivation(tfIdf);
  	
  	//remote log
  	//this.sendRemoteLog("composing annotations",this.manager);
  	
  	//compose annotations and sends them to the annotation repository
    this.composeAnnotations(activation, docURL);


    //Send a report to agency manager
    ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
    IndexReportMessage rm = new IndexReportMessage(docURL);
    msg.setContent(rm.createMsg());
    msg.addReceiver(manager);
    this.send(msg);
    return true;
  }
  
  private Hashtable evaluateTfIdf(String doc, String docURL)
  {
  	//evaluates the frequency term and store data into the
	//proper data structure
	Hashtable tfDoc = this.evaluateTermFrequency(doc);
	
	//debug
	//System.out.println("counted term frequency, adding to the docHash");
	//System.out.println("tfDoc: "+tfDoc);
	
	//then stores the tf hashtable in the cumulative hash
	Hashtable tfHash = null;
	
	//check if there is already an hash for the current language
	if((tfHash=(Hashtable) this.docHash.get(this.lang))!= null)
	{
		this.addTfTerms(tfHash,tfDoc);
	}
	else
	{
		tfHash = new Hashtable();
		this.addTfTerms(tfHash,tfDoc);
	}
	this.docHash.put(this.lang,tfHash);
		
	//for each term evaluates the tfidf value
	Hashtable tfIdfWeights = new Hashtable();
	
	//the mwords
	Iterator enum = this.Mwords.iterator();
	
	while(enum.hasNext())
	{
		//the current mword
		String mword = (String)enum.next();
		
		double tfIdf =0.0;
		
		try
		{
			//gets the tf term
			double tf = ((Double)tfDoc.get(mword)).doubleValue();
		
			//gets the tf sum
			double tfSum = ((Double)tfHash.get(mword)).doubleValue();
		
			//evaluation
			if((tfSum!=0)&&(this.nDocs!=0))
				tfIdf = tf * Math.log(this.nDocs/tfSum);
		
		}
		catch(Exception e)
		{
			//if comes here some error happended thus tfidf = 0;
		}
				
		//storing data
		tfIdfWeights.put(mword,new Double(tfIdf));
	}
	return tfIdfWeights;
  }
  
  private Hashtable evaluateTermFrequency(String doc)
  {
  	//gets the enumeration of MWords to take into account for the
  	//indexing process
    Iterator enumMwords = this.Mwords.iterator();
    
    //prepares the HashTable for storing inDocument occurrences
    Hashtable inDoc = new Hashtable();
    
    //initializing the max-counter
    int max = 0;
    
    //Searching the document for the mword in the synset.
    while (enumMwords.hasNext()) 
    {
    	//gets the current mword
    	String mWord = (String) enumMwords.next();
    	//stems the MWord
    	String stemmed = this.stemmer.stem(mWord);
    	//finds the occurrences of the stemmed MWord
    	int occ = 0;
    	int index = doc.indexOf(stemmed);
    	
    	//remote log
    	//this.sendRemoteLog("Counting occurrences for: "+stemmed,this.manager);
    	
   		while ((index >= 0)&&( index< doc.length())) 
   		{
    		occ++;
    		index += stemmed.length();
    		index = doc.indexOf(stemmed, index);
    		//System.out.println("word: "+stemmed+" index: "+index);
   		}
   		//remove from the text
   		doc = doc.replaceAll(stemmed, "");
   		
   		//store the occurrence of the MWord
   		inDoc.put(mWord,new Integer(occ));
   		
   		//remote log
   		//this.sendRemoteLog("found: "+occ+" occurrences",this.manager);
   		
   		//evaluates the maximum number of occurrences
   		if(occ > max) max = occ;
    }
    
    //reweights the occurrence hashtable, thus obtaining the
    //term frequency hashtable:
    
    //the mwords
    enumMwords = this.Mwords.iterator();
    
    while(enumMwords.hasNext())
    {
    	//gets the current multiword
    	String mword = (String)enumMwords.next();
    	//the new tf value
    	double tf = 0.0;
    	//evaluates the new tf value
    	try
		{
    	if( max!=0 )
    	  tf = ((Integer)inDoc.get(mword)).doubleValue()/(double)max;
		}
    	catch (Exception e)
		{
    		//if comes here the mword is not present in the document, thus tf =0.0
		}
    	//stores the new value
    	inDoc.put(mword,new Double(tf));
    }
    
    //debug
    //System.out.println("InDoc: "+inDoc.toString());
    
    return inDoc;
    
  }
  
  private void addTfTerms(Hashtable tfHash, Hashtable tfDoc)
  {
  	//gets the mwords iterator
  	Iterator iter = this.Mwords.iterator();
  	
  	//for all mwords sum the tf value
  	while(iter.hasNext())
  	{
  		//the mword as literal
  		String mword = (String) iter.next();
  		
  		double tfSum = 0.0;
  		
  		//gets the current tfSum value
  		try
		{
  			tfSum = ((Double)tfHash.get(mword)).doubleValue();
		}
  		catch(Exception e)
		{
  			//if there is a null pointer exception the mword is not yet stored
  			//into the hash, therefore tfSum = 0.0
		}
  		
  		double tfCur = 0.0;
  		
  		//gets the new tfDoc value
  		try
		{
  			tfCur = ((Double)tfDoc.get(mword)).doubleValue();
		}
  		catch(Exception e)
		{
  			//if there is a null pointer exception the mword is not yet stored
  			//into the hash, therefore tfDoc = 0.0
		}
  		
  		//computing the new tfSum value
  		tfSum += tfCur;
  		
  		//storing the tfSum
  		tfHash.put(mword,new Double(tfSum));
  		
  	}
  	
  	//increments the document counter
  	this.nDocs++;
  	
  	return;
  }

  /**
   * The actual method for annotation composition
   * @param t Hashtable of the M-Words collection
   * @param URL the URL of the document, used as a term of an Annotation
   */
  private void composeAnnotations(Hashtable activation, String URL)
  {
  	//composing the annotation
  	
  	//the document is not fragmented therefore the xpath is fixed
  	String xPath = "/html/body";
  	
  	//the author is H-DOSE
  	String author = "default";
  	
  	//the annotation type is default
  	String annType = "default";
  	
  	//debug
  	System.out.println("Activation: "+activation.toString());
  	
  	//extracting the keys enumeration
  	Enumeration enum = activation.keys();
  	
  	//extracting concepts URIs
  	String topicURIs [] = new String [activation.size()];
  	
  	//extracting weights
  	Double weights[] = new Double [activation.size()];
  	
  	//counter
  	int i=0;
  	//extracting the weights
  	while (enum.hasMoreElements())
  	{
  		//the key
  		String topicURI = (String) enum.nextElement();
  		
  		//storing uris
  		topicURIs[i] = topicURI;
  		
  		//storing the weights in the right positions
  		weights[i] = (Double) activation.get(topicURI);
  		
  		//update counter
  		i++;
  	}
  	
  	//remote logging
  	this.sendRemoteLog("Sending annotation for"+URL+"\nvalues:"+activation.toString(),this.manager);
  	//Send to Annotation WS
    try 
	{
    	//creates a new QName identifier for the annotation service
    	// "http://localhost:8080/"
    	QName op = new QName(this.serverAnnoWS, "addAnnotations");

    	//creating service factory
    	ServiceFactory factory = ServiceFactory.newInstance();
	
    	//defining the service
    	Service service = factory.createService(new QName("AnnoRepositoryWSService"));
	
    	//defining the service port
    	QName port = new QName("AnnoRepositoryWS");
	
    	//creating the call object
    	Call call = service.createCall(port);
	
    	//sets the endpoint for the call
    	call.setTargetEndpointAddress(this.endpointAnnoWS);

    	//Setting up the params
    	Object params[] = new Object[6];
    	params[0] = URL;
    	params[1] = xPath;
    	params[2] = topicURIs;
    	params[3] = weights;
    	params[4] = author;
    	params[5] = annType;

    	//sets the operation to be required by the call
    	call.setOperationName(op);
    	Boolean ret = (Boolean) call.invoke(params);
	}
    catch (Exception ex) 
	{
    	this.sendRemoteLog("!! Calling AnnoWS: " + ex.toString(), this.manager);
	}
  }

  /**
   * Evaluates the activation of each ontology concept in consequence
   * of the indexing process
   */
  private Hashtable evaluateConceptActivation(Hashtable tfIdf) 
  {
  		//builds the activation Hashtable
  		Hashtable activation = new Hashtable();
  		
  		//the normalization factor
  		double norm = 0.0;
  		
  		//for each concept, evaluates the tfIdf contribution
  		//and stores the result in the activation Hashtable
  		
  		//the concepts
  		Enumeration enum = this.synWrapper.getConcepts();
  		
  		while(enum.hasMoreElements())
  		{
  			//the concept URI
  			String conceptURI = (String)enum.nextElement();
  			
  			//gets the Synset
  			Synset syn = this.synWrapper.getSynset(conceptURI,this.lang);
  			
  			//gets the MWords
  			MWord mwords[] = syn.getMWords();
  			
  			//unnormalized activation for the concept
  			double unAct = 0.0;
  			
  			//for each mword evaluates the tfIdf contribution
  			for(int i=0; i<mwords.length; i++)
  			{
  				//the literal value
  				String mword = mwords[i].getMWord();
  				
  				//debug
  				//System.out.println("word: "+mword);
  				
  				//the weight
  				//double weight = mwords[i].getWeight();
  				double weight = 1.0;
  				
  				//the mword tfidf
  				double wTfIdf = ((Double)tfIdf.get(mword)).doubleValue();
  				
  				//debug
  				//System.out.println("wTfIdf. "+wTfIdf);
  				
  				//computing the contribution
  				unAct += weight*wTfIdf;
  			}
  			
  			//debug
  			//System.out.println("unAct: "+unAct);
  			
  			//updating the normalization value
  			norm += unAct;
  			
  			// debug
  			//System.out.println("norm: "+norm);
  			
  			//storing the un-normalized activation
  			activation.put(conceptURI,new Double(unAct));
  		}
  		
  		//normalization
  		enum = activation.keys();
  		
  		while(enum.hasMoreElements())
  		{
  			//the conceptURI as literal
  			String conceptURI = (String)enum.nextElement();
  			
  			//updating the stored value
  			
  			//the current value
  			double value = ((Double)activation.get(conceptURI)).doubleValue();
  			
  			//the new value
  			if(norm!=0.0)
  				value /= norm;
  			else
  				value=0.0;
  			
  			//storing the new value
  			activation.put(conceptURI,new Double(value));
  		}
  		
  		//ok, activation stored, return the activation hashtable
  		return activation;
  }

  /**
   * Reads in memory the a lookUpTable from Sysnsets if its expirationDate is
   * still valid..
   * 
   * @param f
   *            XML file containing the Synsets
   * @return true on success, false otherwise
   */
  private boolean acquireSynsets(String content) {
		try 
		{
			//getting the byte buffer for the content
			byte buffer[] = content.getBytes();
			
			//getting the InputStream for the content
			InputStream inStream = new ByteArrayInputStream(buffer);
			
			//building a new synset wrapper
			this.synWrapper = new SynsetWrapperImpl();
			
			//parsing the synset message
			this.synWrapper.parseSynset(inStream);
			
			//changing the current information date
			this.expirationDate = this.synWrapper.getExpiration();
			
			//****** building the MWords Hashset ******
			
			//gets the language
			Set langSet = this.synWrapper.getLanguages();
			
			//gets the first (and the only one by construction)
			this.lang = (String)(langSet.toArray())[0];
			
			//get the synsets
			Enumeration enum = this.synWrapper.getConcepts();
			
			//for each concept extracts the mwords
			while(enum.hasMoreElements())
			{
				//the concept uri
				String conceptURI = (String)enum.nextElement();
				
				//gets the synset for that uri
				Synset syn = this.synWrapper.getSynset(conceptURI,this.lang);
				
				//extracts the mwords
				MWord mwords[] = syn.getMWords();
				
				//stores the mwords in the set
				for(int i = 0; i < mwords.length; i++)
				{
					//stores only the literal value of the MWords
					this.Mwords.add(mwords[i].getMWord());
				}
				
			}
			

		} catch (Exception exc) 
		{
			//if something goes wrong, send an error log and skip
			this.sendRemoteLog("!! AcquireSynsets -2: " + exc.toString(),
					manager);
			return false;
		}
		return true;
	}

}