package it.polito.elite.doseWS.agency;

import jade.core.Agent;
import jade.domain.DFService;
import jade.domain.FIPAAgentManagement.*;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.core.AID;
import jade.lang.acl.ACLMessage;
import it.polito.elite.doseWS.agency.messages.RemoteLoggingMessage;

/**
 *
 * <p>Title: Dose Agency </p>
 * <p>Description: Base abstract class to be extended by DOSE agents</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author alessio.bosca@polito.it
 */
public abstract class DOSEAgent extends Agent
{
  /**
   * <p>Description: interpret message and react properly to its content</p>
   * @param msg The message to be read
   */
  public void handleMessage(ACLMessage msg)
  {}

  boolean register(String serviceDescription)
  {
    ServiceDescription sd = new ServiceDescription();
    sd.setType(serviceDescription);
    sd.setName(getLocalName());
    DFAgentDescription dfd = new DFAgentDescription();
    dfd.setName(getAID());
    dfd.addServices(sd);

    try {
      DFService.register(this, dfd);
    }
    catch (FIPAException fe) {
      System.out.println(fe);
      return false;
    }
    return true;
  }
  /**
   * Sends a remote log to the agent deputed to the remote Logging
   * @param log
   * @param manager
   */
  void sendRemoteLog(String log, AID manager)
  {
    ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
    msg.addReceiver(manager);
    RemoteLoggingMessage rlog = new RemoteLoggingMessage(this.getLocalName(),
	this.here().getName(), log);
    msg.setContent(rlog.createMsg());
    this.send(msg);
  }

  /**
   * Deregister from the DF
   */
  protected void takeDown()
  {
    try {
      DFService.deregister(this);
    }
    catch (Exception e) {}
  }

}