package it.polito.elite.doseWS.agency.behaviours;

import jade.core.behaviours.*;
import it.polito.elite.doseWS.agency.AgencyManager;
import it.polito.elite.doseWS.agency.messages.IsAlive;
import it.polito.elite.doseWS.agency.*;
import jade.lang.acl.ACLMessage;
import jade.core.AID;
import java.util.HashSet;
import java.util.Hashtable;
/**
 *
 * <p>Title: CeckAgencyEmployees </p>
 * <p>Description: Parts of the autonomic features of the platform. It sends an
 * isAlive message to each AID present in Annotators,
 *  Filters and SynsetManagers HashSet collectors of the AgencyManager agents who
 * schedules this behaviour. Such collectors are managed and filled by the manager
 * itself. See subscribeServiceRegistrations method of AgencyManager for details.
 * After sending this message it further schedules a CountEmployeesNumber (extends
 * WakerBehaviour) that its executed after a certain delay time in order to get the
 * AIDs of the not responding agents. The manager backs up such message receiver agent AIDs
 * on its internal collector PendingEmployees Hashtable. From this collector the
 * replyng agents (who therefore are still alive and running)are removed by the
 * manager itself. </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author alessio.bosca@polito.it
 */
public class CeckAgencyEmployees extends OneShotBehaviour
{
  final long wakeUp=60000;
  AgencyManager myAgent;

  public CeckAgencyEmployees(AgencyManager myAgent)
  {
    this.myAgent = myAgent;
  }

  /**
   *
   */
  public void action()
  {
    IsAlive alive = new IsAlive();
    myAgent.PendingEmployees = new Hashtable(this.myAgent.Annotators.size() +
					   this.myAgent.Filters.size() +
					   this.myAgent.SynsetManagers.size() );
    ACLMessage msg = new ACLMessage(ACLMessage.QUERY_IF);
    this.addReceivers(msg, this.myAgent.Annotators,AnnotationAgent.SERVICE);
    this.addReceivers(msg, this.myAgent.Filters,FilterAgent.SERVICE);
    this.addReceivers(msg, this.myAgent.SynsetManagers,SynsetManager.SERVICE);
    msg.setContent(alive.createMsg());
    this.myAgent.send(msg);
    this.myAgent.addBehaviour(new CountEmployeesNumber(this.myAgent,this.wakeUp));
  }

  private void addReceivers(ACLMessage msg, HashSet v, String service)
  {
    AID[] employee = new AID[1];
    employee = (AID[]) v.toArray(employee);
    for (int i = 0; i < employee.length; i++) {
      msg.addReceiver(employee[i]);
      myAgent.PendingEmployees.put(employee[i],service);
    }
  }

}