package it.polito.elite.doseWS.agency.behaviours;

import it.polito.elite.doseWS.agency.DOSEAgent;
import jade.core.behaviours.*;
import jade.domain.DFService;
import jade.domain.FIPAAgentManagement.*;
import java.util.HashSet;

/**
 *
 * <p>Title: GetAgentProviders </p>
 * <p>Description: It retrieves and keep up to date the list of agents providing
 * a certain service. It bakes up the AIDs in a proper HashSet collector.</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author alessio.bosca@polito.it
 */
public class GetAgentProviders extends Behaviour
{
  DOSEAgent myAgent;
  HashSet results;
  DFAgentDescription dfd;
  String serviceDescription;
  long period;
  boolean finished;

  /**
   * Base Constructor
   * @param agent The DOSEAgent scheduling this behaviour
   * @param period Amount of millisec between two checking phases
   * @param serviceDescription The description of the service exported by the searched agents
   * @param results The collections where the AIDs are stored
   */
  public GetAgentProviders(DOSEAgent agent, long period,
                           String serviceDescription,
                           HashSet results)
  {
    super(agent);
    this.myAgent = agent;
    this.results = results;
    this.dfd = new DFAgentDescription();
    //Composes the service description
    ServiceDescription sd = new ServiceDescription();
    sd.setType(serviceDescription);
    this.dfd.addServices(sd);
    this.finished=false;
    this.period = period;
    this.serviceDescription=serviceDescription;
  }

  public boolean done(){ return this.finished; }

  /**
   * It request to the DF an AIDs list of agents providing the requested service
   * and it stores them in the results collector of the scheduling agent.
   * It cycles until it gets at least one AID, then it stops and schedules a
   * further check via a ScheduleNextProvidersRefresh behaviour. keeping in this
   * way the list up to date
   */
  public void action()
  {
    try {
      this.results.clear();
      DFAgentDescription[] result = DFService.search(myAgent, dfd);
      for (int i = 0; i < result.length; i++) {
        //Stores the obtained agent adresses
        this.results.add(result[i].getName());
      }
      if (this.results.size() > 0) {
        //Schedules a new check in the furure via ScheduleNextProvidersRefresh
        this.myAgent.addBehaviour(new ScheduleNextProvidersRefresh(this.myAgent,
            this.period, this.serviceDescription, this.results));
        finished=true;
      }
      else
        block(2000);
    }
    catch (Exception ex) {
      System.out.println(ex.toString());
      block(10000);
    }
  }

}