package it.polito.elite.doseWS.agency.behaviours;

import jade.core.behaviours.*;
import it.polito.elite.doseWS.agency.*;
import jade.lang.acl.ACLMessage;
import it.polito.elite.doseWS.agency.messages.Die;
import java.util.Iterator;
import jade.core.AID;

/**
 *
 * <p>Title: ListenObjectBehaviour </p>
 * <p>Description: It listen requests from the WebService and handles them.
 * The communication between WS and agent world rely on the getO2AObject method
 * present in jade.core.Agent and putO2AObject method present in jade.wrapper.Agent.
 * The wrapper is used by DoseAgencyWS which basically forwards requests to the agency
 * by encapsulating them in a proper RequestObject.</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author alessio.bosca@polito.it
 */
public class ListenObjectBehaviour extends CyclicBehaviour
{
  AgencyManager myAgent;
  RequestObject req;

  public ListenObjectBehaviour(AgencyManager x)
  {
    super(x);
    this.myAgent = x;
  }

  public void action()
  {
    Object obj = this.myAgent.getO2AObject();
    if (obj != null) {
      System.out.println("Received Obj");
      //get the RequestObject
      this.req = (RequestObject) obj;
      //set the RequestObject flag received to true
      this.req.setReceived();
      switch (req.getOperation()) {
	case RequestObject.INDEX_URI:
	  //Get the request
	  String uri=(String)req.getObject();
	  this.myAgent.log("Received external indexing request from WS: "+uri);
	  //Issues the indexing
	  this.myAgent.indexURI(uri);
	  break;

	case RequestObject.DIE:
	  //Create a DIE message
	  ACLMessage msg=new ACLMessage(ACLMessage.REQUEST);
	  Die d=new Die();
	  msg.setContent(d.createMsg());

	  //propagate the message to all Filters
	  Iterator it=this.myAgent.Filters.iterator();
	  while(it.hasNext()){
	    msg.addReceiver((AID)it.next());
	  }
	  //propagate the message to all Annotators
	  it=this.myAgent.Annotators.iterator();
	  while(it.hasNext()){
	    msg.addReceiver((AID)it.next());
	  }
	  //propagate the message to all Synset Managers
	  it=this.myAgent.SynsetManagers.iterator();
	  while(it.hasNext()){
	    msg.addReceiver((AID)it.next());
	  }
	  //send the message
	  this.myAgent.send(msg);
	  //DIE
	  this.myAgent.log("Received stop request from WS");
	  this.myAgent.doDelete();
	  break;

	default:
	  this.myAgent.log("AgencyManager) ListenObj: unknown Operation");
      }
    }
    else {
      block(2000);
    }

  }

}