package it.polito.elite.doseWS.agency.behaviours;

/**
 * <p>Title: RequestObject </p>
 * <p>Description: An object used to encapsulate the requests from the WS 'object'
 * world and send it to the 'agent' world in an apposite agent queue. (The agent must enable
 * this feature via setEnabledO2ACommunication method of jade.core.Agent) It provides
 * get/set methods for: storing request and response (via setObject, getObject);
 * phases synchronization (via setReceived, isReceived and setCompleted,isCompleted);
 *  specification of Operation type (via setOperation, getOperation).
 * This last features relies on a list of static costants defined in the class which
 * specifies the operation allowed, conceptually mapping them on the operations
 * exported by DoseAgencyWS to external users.</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author alessio.bosca@polito.it
 */

public class RequestObject
{
  public final static int INDEX_URI = 1, DIE = 2;
  private int operation;
  private boolean completed, received;
  private Object theObj;

  public RequestObject()
  {
    received = false;
    completed = false;
    theObj = null;
    operation = -1;
  }

  public void setReceived()
  {
    received = true;
  }

  public boolean isReceived()
  {
    return received;
  }

  public void setCompleted()
  {
    completed = true;
  }

  public boolean isCompleted()
  {
    return completed;
  }

  public void setOperation(int i)
  {
    operation = i;
  }

  public int getOperation()
  {
    return operation;
  }

  public void setObject(Object o)
  {
    theObj = o;
  }

  public Object getObject()
  {
    return theObj;
  }

}
