package it.polito.elite.doseWS.agency.messages;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.io.*;
import org.xml.sax.*;

/**
 *
 * <p>Title: AnnotationOfferMessage </p>
     * <p>Description: Subscribe to an Annotation prerequest in a given language. It
 * specifies the expiration date and its current requests queue size</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author alessio.bosca@polito.it
 * @version 3.0
 */
public class AnnotationOfferMessage extends Message
{
  static final String ID = "AnnoOffer";
  Date date;
  int runnings;

  public AnnotationOfferMessage(Date date, int runnings)
  {
    this.date = date;
    this.runnings = runnings;
  }

  public String createMsg()
  {
    //format the date
    SimpleDateFormat df = (SimpleDateFormat) DateFormat.getDateTimeInstance();
    df.applyPattern("yyyy:MM:dd:hh:mm:ss:S:Z");
    //writes the message text
    return new StringBuffer("<MSG id=\"" + AnnotationOfferMessage.ID + "\">").
        append("<EXPIRATION>").append(df.format(date)).append("</EXPIRATION>").
        append("<RUNNINGS>").append(runnings).append("</RUNNINGS>").append(
        "</MSG>").toString();
  }

  public static String getID()
  {
    return AnnotationOfferMessage.ID;
  }

  public static Message parse(String msg)
  {
    AnnotationOfferMessage resp;
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      //Reader for the msg
      StringReader reader = new StringReader(msg);
      //InputSource for the msg
      InputSource source = new InputSource(reader);
      // parses the xml
      Document doc = builder.parse(source);
      Element elemRoot = doc.getDocumentElement();
      //get the expiration date
      Element expiration = (Element) elemRoot.getElementsByTagName("EXPIRATION").
          item(0);
      String expirationStr = ( (org.w3c.dom.Text) expiration.getFirstChild()).
          getData();
      //parse the date according a given format
      SimpleDateFormat df = (SimpleDateFormat) DateFormat.getDateTimeInstance();
      df.applyPattern("yyyy:MM:dd:hh:mm:ss:S:Z");
      Date d = df.parse(expirationStr);
      //get the queue size and parses it into an int
      Element runnings = (Element) elemRoot.getElementsByTagName("RUNNINGS").
          item(0);
      String runningStr = ( (org.w3c.dom.Text) runnings.getFirstChild()).
          getData();
      int r = Integer.parseInt(runningStr);
      resp = new AnnotationOfferMessage(d, r);
    }
    catch (Exception ex) {
      System.out.println(ex);
      return null;
    }
    return resp;
  }

  public Date getDate()
  {
    return date;
  }

  public int getRunnings()
  {
    return runnings;
  }

}