package it.polito.elite.doseWS.agency.messages;

import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.io.*;
import org.xml.sax.*;

/**
 *
 * <p>Title: AnnotationRequestMessage </p>
 * <p>Description: Annotation Request specifing a stemmed text, its language and
 * the original URL of the document.</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author alessio.bosca@polito.it
 */
public class AnnotationRequestMessage extends Message
{
  static final String ID = "AnnoReq";
  String text, language, url;

  public AnnotationRequestMessage(String text, String lang, String url)
  {
    this.text = text;
    this.language = lang;
    this.url = url;
  }

  public String createMsg()
  {
    //The URI is in CDATA format in order to get rid of ? and & stuff
    return new StringBuffer().append("<MSG id=\"" +
                                     AnnotationRequestMessage.ID + "\">").
        append("<URL>").append("<![CDATA[" + this.url + "]]>").append("</URL>").
        append("<TEXT lang=\"" + language +
        "\">").append(text).append("</TEXT>").append("</MSG>").
        toString();
  }

  public static String getID()
  {
    return AnnotationRequestMessage.ID;
  }

  public static Message parse(String msg)
  {
    AnnotationRequestMessage resp;
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      //Reader for the msg
      StringReader reader = new StringReader(msg);
      //InputSource for the msg
      InputSource source = new InputSource(reader);
      // parses the xml
      Document doc = builder.parse(source);
      Element elemRoot = doc.getDocumentElement();
      Element elm = (Element) elemRoot.getElementsByTagName("TEXT").item(0);
      //get language attribute
      String lang = elm.getAttribute("lang");
      //get the clean stemmed text
      String text = ( (org.w3c.dom.Text) elm.getFirstChild()).getData();
      //get the url
      elm = (Element) elemRoot.getElementsByTagName("URL").item(0);
      String url = ( (org.w3c.dom.Text) elm.getFirstChild()).getData();
      //constructs the message instance
      resp = new AnnotationRequestMessage(text, lang, url);
    }
    catch (Exception ex) {
      System.out.println(ex);
      return null;
    }
    return resp;
  }

  public String getLanguage()
  {
    return language;
  }

  public String getText()
  {
    return text;
  }

  public String getURL()
  {
    return url;
  }

}