package it.polito.elite.doseWS.agency.messages;

import java.io.*;
import javax.xml.parsers.*;

import org.w3c.dom.*;
import org.xml.sax.*;

/**
 *
 * <p>Title: IndexRequestMessage </p>
 * <p>Description: An explicit request of indexing. It reports the document url.</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author alessio.bosca@polito.it
 */
public class IndexRequestMessage extends Message
{
  final static String ID = "IndexReq";
  public String requestedURL;

  /**
   * Base Constructor
   * @param url Document to be indexed
   */
  public IndexRequestMessage(String url)
  {
    this.requestedURL = url;
  }

  public String createMsg()
  {
    //The URI is in CDATA format in order to get rid of ? and & stuff
    return new StringBuffer("<MSG id=\"" + IndexRequestMessage.ID + "\">").
        append("<URL>").append("<![CDATA["+this.requestedURL+"]]>").append("</URL>").append("</MSG>").
        toString();
  }

  public static String getID()
  {
    return IndexRequestMessage.ID;
  }

  public static Message parse(String msg)
  {
    IndexRequestMessage resp;
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      //Reader for the msg
      StringReader reader = new StringReader(msg);
      //InputSource for the msg
      InputSource source = new InputSource(reader);
      // parses the xml
      Document doc = builder.parse(source);
      Element elemRoot = doc.getDocumentElement();
      //gets the URL
      Element url = (Element) elemRoot.getElementsByTagName("URL").item(0);
      String urlStr = ( (org.w3c.dom.Text) url.getFirstChild()).getData();
      resp=new IndexRequestMessage(urlStr);
    }
    catch (Exception ex) {
      System.out.println(ex);
      return null;
    }
    return resp;
  }

  public String getRequestedURL()
 {
   return requestedURL;
 }


}
