/*
 * Created on Oct 11, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package it.polito.elite.doseWS.agency.messages;

import java.io.StringReader;
import java.util.Hashtable;
import java.util.Set;
import java.util.HashSet;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Iterator;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.*;
import org.xml.sax.InputSource;

import it.polito.elite.doseWS.include.MWord;
import it.polito.elite.doseWS.include.Synset;
import it.polito.elite.doseWS.include.LearningPair;

/**
 * @author bonino
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class LearningMessage extends Message
{

  private Set toLearn;
  private Hashtable toUpdate;
  private static final String ID = "learning";

  public LearningMessage()
  {
    this.toLearn = new HashSet();
    this.toUpdate = new Hashtable();
  }

  /**
   * Conctructs a learning message starting from an hashtable of mword
   * and a vector of cword. Each base element of both collections is a LearningPair
   * composed by a term and and its occurrence within the document
   * @param mwords Hashtable of mword occurences grouped by concepts
   * @param cwords Vector of new terms with teir occurrence within the document
   * @return
   */
  public String createMsg(Hashtable mwords, HashSet cwords, String language)
  {
    Enumeration e = mwords.keys();
    StringBuffer buff = new StringBuffer("<?xml version=\"1.0\"?><MSG id=\"" + LearningMessage.ID +
                                         "\" lang=\""+language+"\">");
    while (e.hasMoreElements()) {
      String concept = (String) e.nextElement();
      buff=buff.append("<CONCEPT uri=\"" + concept + "\">");
      Vector mw = (Vector) mwords.get(concept);
      for (int i = 0; i < mw.size(); i++) {
        LearningPair pair = (LearningPair) mw.elementAt(i);
        buff=buff.append("<MWORD occurrence=\"" + pair.occurrence +"\">").
            append(pair.term).append("</MWORD>");
      }
      buff=buff.append("</CONCEPT>");
    }//end while
    Iterator it=cwords.iterator();
    while(it.hasNext()){
      LearningPair pair = (LearningPair) it.next();
        buff=buff.append("<CWORD occurrence=\"" + pair.occurrence +"\">").
            append(pair.term).append("</CWORD>");

    }
    return buff.append("</MSG>").toString();
  }

  /**
   * @param msg the message to be parsed
   * @return the LearningMessage object corresponding to msg
   */
  public static Message parse(String msg)
  {

    // the LearningMessage object
    LearningMessage learnMsg = new LearningMessage();

    try {
      // getting the document builder Factory
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

      // getting the document builder from the Factory
      DocumentBuilder builder = factory.newDocumentBuilder();

      //Reader for the msg
      StringReader reader = new StringReader(msg);

      //InputSource for the msg
      InputSource source = new InputSource(reader);

      // parses the xml
      Document synDoc = builder.parse(source);

      // extract the element root of the message
      Element root = synDoc.getDocumentElement();

      //extracting the current lang
      String lang = root.getAttribute("lang");

      // extract the concept element
      NodeList concepts = root.getElementsByTagName("CONCEPT");

      // for each concept extracts the mwords to learn
      for (int i = 0; i < concepts.getLength(); i++)
      {
        Element concept = (Element) concepts.item(i);

        //debug
        //System.out.println("i: "+i+" concepts.getLenght(): "+concepts.getLength());
        //System.out.println("Concept: "+concept.getAttribute("uri"));

        //getting the mword list
        NodeList mwordEntries = concept.getElementsByTagName("MWORD");

        //building the mword array
        MWord mwords[] = new MWord[mwordEntries.getLength()];



        //extracting mwords
        for (int j = 0; j < mwordEntries.getLength(); j++) {
          //getting the mword element
          Element mword = (Element) mwordEntries.item(j);

          //extracting the occurrence attribute
          int occurrence = Integer.parseInt(mword.getAttribute("occurrence"));

          //default weight
          double weight = 0.0;

          //extracting the mword
          String word = ( (Text) mword.getFirstChild()).getData();

          //debug
          //System.out.println("MWords: "+word+":"+occurrence+":"+weight);

          //building the MWord object and storing in the array
          mwords[j] = new MWord(word, occurrence, weight);
        }



        //extracting the concept uri
        String conceptURI = concept.getAttribute("uri");

        //build the toUpdate synset
        Synset synToUpdate = new Synset(conceptURI, mwords, lang);

        //storing data
        learnMsg.addToUpdate(synToUpdate);

      }

      //getting the cword list
      NodeList cwordEntries = root.getElementsByTagName("CWORD");

      //extracting cwords
      for (int k = 0; k < cwordEntries.getLength(); k++) {
        //getting the mword element
        Element cword = (Element) cwordEntries.item(k);

        //extracting the occurrence attribute
        int occurrence = Integer.parseInt(cword.getAttribute("occurrence"));

        //default weight
        double weight = 0.0;

        //extracting the mword
        String word = ( (Text) cword.getFirstChild()).getData();

        //debug
        //System.out.println("CWords: "+word+":"+occurrence+":"+weight);

        //building the MWord object and storing in the array
        learnMsg.addToLearn(new MWord(word, occurrence, weight));
      }


    }
    catch (Exception e) {
      return null;
    }

    return learnMsg;
  }

  /**
   * @param synToLearn the synset to be added to the class internal storage
   * for synset participating in learning new terms
   */
  private void addToLearn(MWord mword)
  {
    this.toLearn.add(mword);
  }

  /**
   * @param synToUpdate the synset to be added to the class internal storage
   * for synset participating in updating probabilities of the already
   * available terms
   */
  private void addToUpdate(Synset synToUpdate)
  {
    this.toUpdate.put(synToUpdate.getConcept(), synToUpdate);
  }

  /**
   * @return the Set of concepts to which the mwords to learn are associated
   */
  public Set getConceptsToLearn()
  {
    return this.getConceptsToUpdate();
  }

  /**
   * @return the Set of concepts to which the mwords to update probabilities
   * are associated
   */
  public Set getConceptsToUpdate()
  {
    return this.toUpdate.keySet();
  }

  /**
   * @param concept the concept for which a  synset to learn must be extracted
   * @return the synset to learn associated to that concept
   */
  public Set getToLearnMWords()
  {
    return this.toLearn;
  }

  /**
   * @param concept the concept for which a synset to update probabilities must
   * be extracted
   * @return the synset to update probabilities associated to that concept
   */
  public Synset getToUpdateSynset(String concept)
  {
    return (Synset)this.toUpdate.get(concept);
  }

  public static String getID()
  {
    //returns the message id for recognition
    return LearningMessage.ID;
  }

  /**
   * @return
   */
  public String createAckMsg()
  {
    return new String("<MESSAGE id=\"acknowledgeLearning\" />");
  }

}
