package it.polito.elite.doseWS.agency.messages;

import java.io.*;
import javax.xml.parsers.*;

import org.w3c.dom.*;
import org.xml.sax.*;



/**
 *
 * <p>Title: RemoteLoggingMessage </p>
 * <p>Description: Report a log message and reports also the name of the agent sending
 * the message and the location from which it originates.</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author alessio.bosca@polito.it
 */
public class RemoteLoggingMessage extends Message
{
  final static String ID = "RemoteLog";
  private String log, agent, location;

  public RemoteLoggingMessage(String myAgentName, String location, String log)
  {
    this.log = log;
    this.agent = myAgentName;
    this.location=location;
  }

  public String createMsg()
  {
    //Log Message itself sent as CDATA in order to avoid url related problems
    return new StringBuffer("<MSG id=\"" + RemoteLoggingMessage.ID + "\">").
        append("<AGENT location=\""+location+"\">").append(this.agent).append("</AGENT>").append("<LOG>").
        append("<![CDATA["+this.log+"]]>").append("</LOG>").append("</MSG>").
        toString();
  }

  public static String getID()
  {
    return RemoteLoggingMessage.ID;
  }

  public static Message parse(String msg)
  {
    RemoteLoggingMessage resp;
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      //Reader for the msg
      StringReader reader = new StringReader(msg);
      //InputSource for the msg
      InputSource source = new InputSource(reader);
      // parses the xml
      Document doc = builder.parse(source);
      Element elemRoot = doc.getDocumentElement();
      //get agent name
      Element elm = (Element) elemRoot.getElementsByTagName("AGENT").item(0);
      String agentName = ( (org.w3c.dom.Text) elm.getFirstChild()).getData();
      //get location attribute of agent
      String location=elm.getAttribute("location");
      //get log core message
      elm = (Element) elemRoot.getElementsByTagName("LOG").item(0);
      String log = ( (org.w3c.dom.Text) elm.getFirstChild()).getData();
      //constructs the message
      resp = new RemoteLoggingMessage(agentName,location,log);
    }
    catch (Exception ex) {
      System.out.println(ex);
      return null;
    }
    return resp;
  }

  public String getLog()
  {
    return this.log;
  }

  public String getAgent()
  {
    return this.agent;
  }

  public String getLocation()
  {
    return this.location;
  }

}