package it.polito.elite.doseWS.agency.messages;

import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.io.*;
import org.xml.sax.*;

/**
 *
 * <p>Title: SynsetResponseMessage </p>
 * <p>Description: It send a bunch of synset according to the request. It leaves
 * to the destination agent the task to actually parse it. </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Politecnico di Torino</p>
 * @author alessio.bosca@polito.it
 */
public class SynsetResponseMessage extends Message
{
  static final String ID = "synResp";
  private String content;

  public SynsetResponseMessage(String content)
  {
    this.content=content;
  }

  public String createMsg()
  {
  	StringBuffer msg = new StringBuffer("<?xml version=\"1.0\"?><MSG id=\"" + SynsetResponseMessage.ID + "\">");
  	msg.append(this.content);
	msg.append("</MSG>");
    return  msg.toString();
  }

  public static String getID()
  {
    return SynsetResponseMessage.ID;
  }

  public static Message parse(String msg)
  {
    SynsetResponseMessage resp;
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      //Reader for the msg
      StringReader reader = new StringReader(msg);
      //InputSource for the msg
      InputSource source = new InputSource(reader);
      // parses the xml
      Document doc = builder.parse(source);
      Element elemRoot = doc.getDocumentElement();
      //get the synsets
      Element elem=(Element) elemRoot.getElementsByTagName("SYNSETS").item(0);
      //get the xml sub document
      String content = elem.toString();
      resp=new SynsetResponseMessage(content);
    }
    catch (Exception ex) {
      System.out.println(ex);
      return null;
    }
    return resp;
  }

  /**
   * Return the synset as an XML subtree enclosed by a <SYNSETS> tag. The task of
   * parsing it is delegated to the destination agent
   * @return An XML subdocument as String
   */
  public String getContent()
  {
    return this.content;
  }

}