/*
 * Created on Oct 7, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package it.polito.elite.doseWS.include;

/**
 *  A class for storing mwords contained in each synset that is associated to
 * the concepts of the ontology actually in use. Supports all required functionalities
 * including the ability to store and retrieve the frequency with which the
 * word is seen in the indexed documents
 * 
 * @author <a href="mailto:dario.bonino@polito.it">Dario Bonino</a>
 * <a href="mailto:alessio.bosca@polito.it">Alessio Bosca</a>
 * <a href="mailto:federico.pescarmona@polito.it">Federico Pescarmona</a>
 * <a href="mailto:michele.debandi@polito.it">Michele Debandi</a>
 * @version 1.0
 * @since 11/2004
 * @see The MWordTest class for testing
 * 
 *
 */
public class MWord {

	/**
	 * Constructor
	 * @param mword the literal value for the mword represented by this object
	 * @param frequency the frequency of occurence of this mword into indexed documents
	 * @param the expert given weight for this object
	 */
	private String mword;
	private int frequency;
	private double weight;
	private double probability = 0.0;
	
	/**
	 * Constructor
	 * @param mword the literal value for the mword represented by this object
	 * @param frequency the frequency of occurence of this mword into indexed documents
	 * @param weight the expert given weight for this object
	 */
	public MWord(String mword, int frequency, double weight) {
		
		this.mword = mword;
		this.frequency = frequency;
		this.weight = weight;
	}

	/**
	 * @return the literal value of the mword
	 */
	public String getMWord() {
		return this.mword.trim();
	}

        public String toString() {
                return this.mword+";"+this.frequency+";"+this.weight;
        }
	/**
	 * @return the frequency, i.e., the number of occurrences
	 * of the mword in the set of indexed documents
	 */
	public int getFrequency() {
		return this.frequency;
	}

	/**
	 * @return the weigth given by experts to this mword
	 */
	public double getWeight() {
		return this.weight;
	}

	/**
	 * @param mword the literal value to be stored in this object
	 */
	public void setMWord(String mword) {
		this.mword = mword;
	}

	/**
	 * @param frequency the frequency of occurence of this mword into indexed documents
	 * stored in this object
	 */
	public void setFrequency(int frequency) {
		this.frequency = frequency;
	}

	/**
	 * @param weight the expert given weight of the mword stored in
	 * this object
	 */
	public void setWeight(double weight) {
		this.weight = weight;
	}
	
	/**
	 * 
	 * @param p the temporary probability that this mword object has to
	 * cause a document to be annotated as about the concept to which the mword is related
	 */
	public void setProbability(double p)
	{
		this.probability = p;
	}
	/**
	 * 
	 * @return the temporary probability that this mword object has to
	 * cause a document to be annotated as about the concept to which the mword is related
	 */
	public double getProbability()
	{
		return this.probability;
	}
	
	/**
	 * 
	 * @return the serialized version of this object according to a predefined XML format
	 */
	public String serialize()
	{
		//creating xml serialization for the synset object
		StringBuffer serialized = new StringBuffer();
		
		//creating the start tag
		serialized.append("<MWORD weight=\""+this.probability+"\" occurrence=\""+this.frequency+"\">");
		
		//adding the mword term
		serialized.append(this.mword);
		
		//creating the end tag
		serialized.append("</MWORD>");
		
		return serialized.toString();
	}

}
