package it.polito.elite.doseWS.include;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.regex.Pattern;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.RDF;
import com.hp.hpl.jena.datatypes.xsd.XSDDatatype;
import net.sf.snowball.SnowballProgram;


/**
 * Semantic Mapper based on the use of weighted synsets.
 */
public class SynSemanticMapper implements SemanticMapper {
	private Hashtable syns, stemmedSyns, languageLookupTable, synWords,
			stemmedSynWords, compoundWords;

	private Property weightP, synP;

	private final static String synFolder = "files/syns/";

	private final static String synFile = "syn_";

	/**
	 * Constructor.
	 *
	 */
	public SynSemanticMapper() {
		initLanguages();
		synWords = new Hashtable();
		stemmedSynWords = new Hashtable();
		compoundWords = new Hashtable();
		// costruire i synset (normale e stemmato)
		createSynsets();
		// richiamare l'ontologia

	}

	private void initLanguages() {
		languageLookupTable = new Hashtable();
		languageLookupTable.put("en", "english");
		languageLookupTable.put("it", "italian");
	}

	private void createSynsets() {
		String[] str;
		String temp, language, namespace;
		Model syn, stemmedSyn;
		syns = new Hashtable();
		stemmedSyns = new Hashtable();
		File file, folder;
		File[] files;
		folder = new File(synFolder);
		files = folder.listFiles();
		int k;
		for (k = 0; k < files.length; k++) {
			file = files[k];
			if (file.getName().matches(synFile + "[a-z]*")) {
				language = file.getName().substring(synFile.length());
				System.out
						.println("Loading syns from file: " + file.toString());
				syn = readModelFromDisk(file);
				ResIterator iter = syn.listSubjectsWithProperty(RDF.li(1));
				Resource res = iter.nextResource();
				namespace = res.getNameSpace();
				weightP = syn.getProperty(namespace + "weight");
				synP = syn.getProperty(namespace + "syn");
				syns.put(language, syn);
				stemmedSyn = createStemmedModel(language);
				stemmedSyns.put(language, stemmedSyn);
			}
		}
	}

	private Model readModelFromDisk(File file) {
		Model syn = ModelFactory.createDefaultModel();
		try {
			InputStream in = SynSemanticMapper.class.getClassLoader()
					.getResourceAsStream(file.toString());
			if (in != null) {
				syn.read(new InputStreamReader(in), null); // second argument for resolving relative URIs
			}
		} catch (Exception e) {
			System.err.println("Failed reading model from file "
					+ file.getName() + ": " + e);
		}
		return syn;
	}

	private Model createStemmedModel(String language) {
		Model stemmedSyn = ModelFactory.createDefaultModel();
		Model syn = (Model) syns.get(language);
		TreeSet mw, smw, cw;
		mw = new TreeSet();
		smw = new TreeSet();
		cw = new TreeSet();
		//System.out.println(language);
		ResIterator iter = syn.listSubjectsWithProperty(RDF.li(1));
		Resource res, res1;
		StmtIterator sIter;
		NodeIterator nIter;
		String string;
		String[] words;
		Bag synset, synset1;
		int i, k, size;
		float w;
		Statement stmt, stmt1;
		try {
			while (iter.hasNext()) {
				res = iter.nextResource();
				synset = syn.getBag(res);
				size = synset.size();
				if (size > 0) {
					//System.out.println("\n" + res.toString());
					synset1 = stemmedSyn.createBag(res.toString());
					for (k = 1; k <= size; k++) {
						stmt = synset.getProperty(RDF.li(k));
						res = (Resource) stmt.getObject();
						string = res.getProperty(synP).getString();
						if (isASingleWord(string)) {
							mw.add(string);
							w = res.getProperty(weightP).getFloat();
						/*words = getWords(string.toLowerCase());
						 string = "";
						 for (i = 0;(i < words.length) && (words[i] != null); i++) {
						 if (words[i].indexOf('.') < 0)
						 string = string + " " + stem(language, removeAccent(words[i]));
						 else
						 string = string + " " + words[i];
						 }
						 string = string.trim();*/
						//System.out.println(string);
							string = stem(language, string);
							smw.add(string);
							res1 = stemmedSyn.createResource();
							res1.addProperty(synP, stemmedSyn.createTypedLiteral(
									string, XSDDatatype.XSDstring));
							res1.addProperty(weightP, stemmedSyn
									.createTypedLiteral(String.valueOf(w),
											XSDDatatype.XSDfloat));
							synset1.add(res1);
						} else
							cw.add(string);
					}
				}
			}
		} catch (Exception e) {
			System.err.println("Error creating stemmed model for language "
					+ language + " " + e);
			System.exit(0);
		}
		synWords.put(language, mw);
		stemmedSynWords.put(language, smw);
		compoundWords.put(language,cw);
		return stemmedSyn;
	}

	/*
	 * STEMMER
	 * 
	 * Liberamente tratto da Snowball http://snowball.tartarus.org/ e
	 * utilizzato dietro licenza BSD (vedi
	 * http://www.opensource.org/licenses/bsd-license.html).
	 * 
	 * Copyright (c) 2001, Dr Martin Porter, e (Java) Copyright (c) 2002,
	 * Richard Boulton.
	 */
	/**
	 * Stems a string according to a given language.
	 * 
	 * @param language
	 *            The language to be used for stemming.
	 * @param word
	 *            The string to be stemmed.
	 * @return The stemmed string.
	 */
	private String stem(String language, String word) {
		String stemmed = word;
		if (language.length() == 2)
			language = languageLookupTable.get(language).toString();
		try {
			Class stemClass = Class.forName("net.sf.snowball.ext." + language
					+ "Stemmer");
			SnowballProgram stemmer = (SnowballProgram) stemClass.newInstance();
			Method stemMethod = stemClass.getMethod("stem", new Class[0]);
			stemmer.setCurrent(word);
			stemMethod.invoke(stemmer, new Object[0]);
			stemmed = stemmer.getCurrent();
		} catch (Exception e) {
		}
		return stemmed;
	}
	
	private Hashtable getWeightedOntClasses(String mword, String language) {
		Hashtable ht = new Hashtable();
		String uri;
		float w;
		Statement stmt;
		Model syn = (Model) syns.get(language);
		StmtIterator bagIter;
		StmtIterator iter = syn.listStatements(null, synP, mword.toLowerCase());
		while (iter.hasNext()) {
			stmt = iter.nextStatement();
			w = stmt.getSubject().getProperty(weightP).getFloat();
			if (w > 0) {
				bagIter = syn.listStatements(null, null, stmt.getSubject());
				while (bagIter.hasNext()) {
					uri = bagIter.nextStatement().getSubject().toString();
					if(ht.containsKey(uri))
						w += ((Float) ht.get(uri)).floatValue();
					ht.put(uri,new Float(w));
				}
			}
		}
		syn = (Model) stemmedSyns.get(language);
		mword = stem(language,mword);
		iter = syn.listStatements(null, synP, mword.toLowerCase());
		while (iter.hasNext()) {
			stmt = iter.nextStatement();
			w = stmt.getSubject().getProperty(weightP).getFloat();
			if (w > 0) {
				bagIter = syn.listStatements(null, null, stmt.getSubject());
				while (bagIter.hasNext()) {
					uri = bagIter.nextStatement().getSubject().toString();
					if(ht.containsKey(uri))
						w += ((Float) ht.get(uri)).floatValue();
					ht.put(uri,new Float(w));
				}
			}
		}
		return ht;
	}
	
    /**
	 * Detetects if the given string is a single word or a word compound.
	 * 
	 * @param string
	 *            The input string.
	 * @return true, if the string is a single word. false, in every other
	 *         cases.
	 */
	private boolean isASingleWord(String string) {
		string = string.trim();
		boolean flag = (string.indexOf(" ")) < 0 ? true : false;
		flag = flag || ((string.indexOf("-")) < 0 ? true : false);
		return flag;
	}
	
	private String[] getWords(String text) {
		String[] words = Pattern.compile("^\\w").split(text.toLowerCase().trim());
		return words;
	}
	
	private Hashtable searchCompoundWords(String text, String language) {
		Hashtable ht = new Hashtable();
		TreeSet ts = (TreeSet) compoundWords.get(language);
		Iterator iter = ts.iterator();
		String mw;
		int index = 0, c;
		while(iter.hasNext()) {
			mw = iter.next().toString();
			index = text.indexOf(mw,index);
			if(index >= 0) {
				c = 1;
				if(ht.containsKey(mw))
					c += ((Integer) ht.get(mw)).intValue();
				ht.put(mw, new Integer(c));
			}
		}
		return ht;
	}
	
	private Hashtable searchWords(String[] words, String language, Hashtable counters) {
		// solo le parole stemmate?
		String word, sWord;
		int c;
		TreeSet stemmed = (TreeSet) stemmedSyns.get(language);
		for(int k = 0; k < words.length; k++) {
			word = words[k];
			sWord = stem(language,word);
			if (stemmed.contains(sWord)) {
				c = 1;
				if(counters.containsKey(word))
					c += ((Integer) counters.get(word)).intValue();
				counters.put(word,new Integer(c));
			}
		}
		return counters;
	}

	/**
	 * Maps a string of text in a given language to an Hashtable of weighted
	 * concepts of the ontology.
	 * 
	 * @param text
	 *            A String of text
	 * @param language
	 *            The language code
	 * @return The Hashtable of weighted concepts
	 */
	public Hashtable map(String text, String language) {
		Hashtable weights = new Hashtable();
		Hashtable counters = searchCompoundWords(text,language);
		String[] words = getWords(text);
		counters = searchWords(words, language,counters);
		
		int c;
		float w;
		Enumeration enumC = counters.keys();
		Enumeration enumCl;
		String mword;
		String cl;
		Hashtable classes;
		while(enumC.hasMoreElements()) {
			mword = (String) enumC.nextElement();
			c = ((Integer) counters.get(mword)).intValue();
			classes = getWeightedOntClasses(mword, language);
			enumCl = classes.keys();
			while (enumCl.hasMoreElements()) {
				cl = "";
				w = 0;
				cl = (String) enumCl.nextElement();
				w = c * ((Float) classes.get(cl)).floatValue();
				if ((w != 0) && (!cl.equals(""))) {
					if (weights.containsKey(cl))
						w += ((Float) weights.get(cl)).floatValue();
					weights.put(cl, new Float(w));
				}
			}
		}
		return weights;
	}
}