/*
 * Created on Oct 1, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package it.polito.elite.doseWS.include;

import java.util.Arrays;
import java.util.Vector;

/**
 * A class for storing data relative to the autonomic policies of the
 * Annotation Repository, stores concepts together with their frequencies in terms
 * of indexed resources, for detecting low covered conceptual areas in
 * the ontology
 * 
 *  @author <a href="mailto:dario.bonino@polito.it">Dario Bonino</a>
 * <a href="mailto:alessio.bosca@polito.it">Alessio Bosca</a>
 * <a href="mailto:federico.pescarmona@polito.it">Federico Pescarmona</a>
 * <a href="mailto:michele.debandi@polito.it">Michele Debandi</a>
 *
 * @version 1.0
 * @since 11/2004
 */
public class TopicCoverage {
	
	private TopicFrequency[] topicsAndFrequencies = null;
		
	/**
	 * Constructor
	 * @param array the array of TopicFrquency objects to be stored in this TopicCoverage
	 * @param sort if true causes the TopicFrequencies to be sorted
	 */
	public TopicCoverage(TopicFrequency array[], boolean sort)
	{
		if(sort) Arrays.sort(array,new TopicFrequencyComparator(TopicFrequencyComparator.ASC));
		this.topicsAndFrequencies = (TopicFrequency[])array.clone();
	}
	
	/**
	 * Constructor
	 * @param array the array of TopicFrequency objects to be stored in this TopicCoverage
	 */
	public TopicCoverage(TopicFrequency array[])
	{
		this.topicsAndFrequencies = (TopicFrequency[])array.clone();
	}
	
	/**
	 * Constructor
	 * @param V the Vector of topic frequencies to be stored in this TopicCoverage
	 */
    public TopicCoverage(Vector V) 
    {
    	V.trimToSize();
        this.topicsAndFrequencies = new TopicFrequency[V.size()];
        for(int i=0; i<V.size(); i++) this.topicsAndFrequencies[i]=(TopicFrequency)V.elementAt(i);
    }
        
    /**
     * 
     * @return the mean value for the coverage frequency of topics contained in this object
     */
	public double coverageMean()
	{
		//evaluates the mean frequency for the TopicCoverage object
		int sum = 0;
		for (int i=0; i<this.topicsAndFrequencies.length; i++)
			sum+=this.topicsAndFrequencies[i].getFrequency();
		return (double)sum/(double)this.topicsAndFrequencies.length;
	}
	/**
	 * 
	 * @return the variance value for the coverage frequency of topics contained in this object
	 */
	public double coverageVariance()
	{
		double mean = this.coverageMean();
		double variance = 0.0;
		for(int i=0; i<this.topicsAndFrequencies.length; i++)
		{
			double currentFrequency =(double)this.topicsAndFrequencies[i].getFrequency();
			double error = currentFrequency - mean; 
			variance+=(error*error);
		}
		return variance/(double)(this.topicsAndFrequencies.length-1);
	}
	
	/**
	 * 
	 * @return the 10% low covered topics, i.e., the 10% topics having less resources pointing to
	 * them by means of semantic annotations
	 */
	public String[] getLowCoveredConcepts()
	{
		//extracting the lowest 10% of the topics coverage
		int tenPercent = this.topicsAndFrequencies.length/10;
		//declaring the result array
		String result[] = new String[tenPercent];
		//loading results
		for (int i=0; i<tenPercent; i++)
			result[i]=this.topicsAndFrequencies[i].getTopicURI();
		//returning results
		return result;
	}
	
	/**
	 * 
	 * @return the topics having the coverage value under a given fraction of the coverage mean
	 * computed as:<br><center> coverage < mean - sqrt(variance)</center>
	 */
	public String[] getLowCoveredConceptsM()
	{
		//extracting the low covered concept set
		Vector result = new Vector();
		double mean = this.coverageMean();
		double sigma = Math.sqrt(this.coverageVariance());
		
		//selecting low covered concepts
		if((3*sigma) > (mean/2))
		{
			int i=0;
			int frequency = 0; 
			do
			{
				frequency = this.topicsAndFrequencies[i].getFrequency();
				result.add(this.topicsAndFrequencies[i].getTopicURI());
				i++;
			}
			while((i < this.topicsAndFrequencies.length)&&(frequency <(mean-sigma) ));
		}
		return (String[])result.toArray();
	}
	
}
