/*
 * TopicViewer.java
 *
 * Created on March 15, 2004, 4:55 PM
 */

package it.polito.elite.doseWS.include;

import org.jfree.data.*;
import org.jfree.chart.*;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.BarRenderer;


import java.awt.*;

/**
 *
 * @author  bonino
 */
public class TopicViewer extends javax.swing.JFrame
{
	private String seriesName[]= {"Conceptual Spectrum for: "};
	private String categoryNames[]; 
	private Double categoryData[];
	private DefaultCategoryDataset topicSet;
	private JFreeChart topicChart;
	private ChartPanel panelView;
	private String spectrumURI;
	
	
	/** Creates new form TopicViewer */
	public TopicViewer()
	{
		initComponents();
		initCustomized();
		//createChart();
		
	}
	public void setNames(String[] names)
	{
		try
		{
			//creates the vector of category names
			this.categoryNames = new String [names.length];
			
			//for each topic name, purges the namespace
			for(int i=0; i < names.length; i++)
			{
				String fullName = names[i];
				int index = fullName.indexOf('#');
				String name = fullName.substring(index+1,fullName.length());
				this.categoryNames[i] = "   "+name;
			}
		}
		catch(Exception e)
		{
			System.out.println(e);
		}
	}
	public void setData(Double data[])
	{
		try
		{
			//creates the vector of topic weights
			this.categoryData = new Double [this.categoryNames.length];
			
			//System.out.println("Data size: "+data.length);
			
			for(int i=0; i<data.length; i++)			
			{
				//System.out.println("Inserted value: "+data[i].doubleValue());
				this.categoryData[i]=data[i];
			}
		}
		catch(Exception e)
		{
			System.out.println("topicViewer [ERROR]: "+e);
		}
	}
	
	public void setCurrentSpectrumURI(String uri)
	{
		this.spectrumURI = new String(uri);
	}

	public void initCustomized()
	{
		//init the application frame size at a convenient one
		this.setSize(700,470);
		this.setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource("/Logo_elite.GIF")));
	}
	
	public void createChart()
	{
		//defines the dataset to be visualized
		this.topicSet = new DefaultCategoryDataset();
		
		double max=0;
		for(int i=0; i<this.categoryData.length; i++)
			if(this.categoryData[i].doubleValue() > max) max = this.categoryData[i].doubleValue();
		
		//stores data into the dataset
		for(int i=0; i<this.categoryData.length; i++)
			if(this.categoryData[i].doubleValue()>(0.1*max))
				this.topicSet.addValue(this.categoryData[i],this.categoryNames[i],this.categoryNames[i]);
		//the bar renderer
		BarRenderer renderer = new BarRenderer();
		renderer.setDrawBarOutline(true);
		renderer.setMaxBarWidth(1.0);
		renderer.setMinimumBarLength(0.3);
		
		
		//the category axis
		CategoryAxis topicAxis= new CategoryAxis("Topics");
		topicAxis.setTickLabelFont(new Font("Courier",Font.PLAIN,10));
		topicAxis.setUpperMargin(0.05);
		topicAxis.setLowerMargin(0.05);
		topicAxis.setMaxCategoryLabelWidthRatio((float)0.2);
		topicAxis.setCategoryLabelPositionOffset(2);
		topicAxis.setCategoryMargin(0.0);
		topicAxis.setCategoryLabelPositions(CategoryLabelPositions.STANDARD);
		topicAxis.setFixedDimension(0.6);
		
		//the number axis
		NumberAxis valueAxis = new NumberAxis();
		valueAxis.setTickLabelFont(new Font("Courier",Font.PLAIN,10));
		valueAxis.setFixedDimension(0.6);
		//valueAxis.setRange(0.0,1.0);
		
		//the bar plot
		CategoryPlot plot = new CategoryPlot(this.topicSet,topicAxis,valueAxis,renderer);
		plot.setDataAreaRatio(1.2);

		
		
		//setting plot orientation
		plot.setOrientation(PlotOrientation.HORIZONTAL);
		
		topicChart = new JFreeChart(plot);
		topicChart.setTitle("Conceptual Spectrum for: "+this.spectrumURI);
		
		//prepares a new panel for the chart
		panelView = new ChartPanel(topicChart);
		this.setContentPane(panelView);
	}
	
	public int saveChart(String chartFilename)
	{
		try
		{
			ChartUtilities.saveChartAsPNG(new java.io.File(chartFilename), this.topicChart, 1024,768 );
			return 0;
		}
		catch (Exception e)
		{
			return -1;
		}
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    private void initComponents()//GEN-BEGIN:initComponents
    {

        getContentPane().setLayout(new java.awt.FlowLayout());

        setTitle("DOSE Conceptual Spectrum Viewer");
        setForeground(java.awt.Color.lightGray);
        addComponentListener(new java.awt.event.ComponentAdapter()
        {
            public void componentHidden(java.awt.event.ComponentEvent evt)
            {
                formComponentHidden(evt);
            }
        });
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                exitForm(evt);
            }
        });

        pack();
    }//GEN-END:initComponents

	private void formComponentHidden(java.awt.event.ComponentEvent evt)//GEN-FIRST:event_formComponentHidden
	{//GEN-HEADEREND:event_formComponentHidden
		// Add your handling code here:
	}//GEN-LAST:event_formComponentHidden
	
	/** Exit the Application */
	private void exitForm(java.awt.event.WindowEvent evt)//GEN-FIRST:event_exitForm
	{
		System.exit(0);
	}//GEN-LAST:event_exitForm
	
	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[])
	{
		TopicViewer graph= new TopicViewer();
		graph.createChart();
		graph.show();
		//graph.repaint();
		
	}
	
	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
	
}
