/*
 * Created on Sep 9, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package it.polito.elite.doseWS.kernel;

import java.io.InputStream;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;
import java.util.Enumeration;
import java.rmi.RemoteException;
import javax.xml.rpc.*;
import javax.xml.rpc.ServiceException;
import java.net.MalformedURLException;
import javax.xml.namespace.QName;

//axis imports
//import org.apache.axis.client.*;
//import org.apache.axis.configuration.*;

//doseWS imports
import it.polito.elite.doseWS.wrappers.DBWrapper;
import it.polito.elite.doseWS.wrappers.DBWrapperImpl;
import it.polito.elite.doseWS.include.Spectrum;
import it.polito.elite.doseWS.include.SpectrumImpl;


/**
 * The Annotation Repository implementation. <br>
 * The annotation repository stores all semantic annotations relating documents, or
 * specific substructures of them to ontology concepts. The repository has persistence on 
 * a DB by istantiating a proper DBWrapper.
 * 
 * @author <a href="mailto:dario.bonino@polito.it">Dario Bonino</a>
 * <a href="mailto:alessio.bosca@polito.it">Alessio Bosca</a>
 * <a href="mailto:federico.pescarmona@polito.it">Federico Pescarmona</a>
 * <a href="mailto:michele.debandi@polito.it">Michele Debandi</a>
 *
 * @see the class AnnoRepositoryImplTest for a testing class
 * @version 1.0
 * @since 11/2004
 */
public class AnnoRepositoryImpl implements AnnoRepository {

	private static DBWrapper storage;
	//	default endpoint
	private static String endpoint = "http://localhost:8080/axis/ExpanderWS.jws";
	private static String server = "http://localhost:8080/";
	private static boolean ready = false;
	
	private static boolean debug=true;
	
	/**
	 * void constructor web-service ready
	 *
	 */
	public AnnoRepositoryImpl ()
	{

	}
	
	/**
	 * The static initialization method for the annotation repository, sets all parameters
	 * required by the repository during the runtime phase
	 *  
	 * @param endpoint the endpoint of the Expander Web Service
	 * @param server the server onto which the Expander runs
	 * @param dbDriver the db driver that should be used by the internal DBWrapper
	 * @param dbURI the uri of the DB server
	 * @param dbUser the username for accessing the db
	 * @param dbPassword the  password for the db, if required
	 * @throws Exception The error that caused the initialization failure
	 */
	public static void initAnnoRepository(String endpoint, String server,String dbDriver,String dbURI, String dbUser, String dbPassword) throws Exception
	{
		storage = new DBWrapperImpl(dbDriver, dbURI, dbUser, dbPassword);
		AnnoRepositoryImpl.endpoint = endpoint;
		AnnoRepositoryImpl.server = server;
		AnnoRepositoryImpl.ready = true;
	}
	
	/**
	 * Flag for checking if the service has been correctly initialized
	 * @return the status of the service, true if initialized otherwise false
	 */
	public static boolean isReady()
	{
		return AnnoRepositoryImpl.ready;
	}
	
	/** 
	 * 
	 * Removes a semantic annotation from the repository
	 * 
	 * @param uri the uri of the document for which annotations should be removed
	 * @param xPath the xPath of the specific substructure for which annotations must be removed
	 * @param author the author of the annotations
	 * @return 0 if the annotations have been removed successfully
	 * @see it.polito.elite.doseWS.kernel.AnnoRepository#remove(java.lang.String, java.lang.String, java.lang.String)
	 */
	public int remove(String uri, String xPath, String author)throws Exception 
	{
		//tries to delete the given spectrum
		int errorCode = storage.deleteSpectrum(uri,xPath,author,null);
		//checks the error code
		if (errorCode < 0)
		{
			throw new Exception(((DBWrapperImpl)storage).dbStatus());
		}
		//tries to delete the corresponding expanded spectrum
		errorCode = storage.deleteXSpectrum(uri,xPath,author,null);
		//checks the error code
		if (errorCode < 0)
		{
			throw new Exception(((DBWrapperImpl)storage).dbStatus());
		}
		return 0;
	}

	/**
	 * Adds a series of semantic annotations corresponding to a conceptual spectra, to the
	 * annotation repository
	 * 
	 * @param fragmentURI the URI of the document for which annotations should be added in the repository
	 * @param fragmentXPath the XPath of the specific substructure of the document to which annotations are referred
	 * @param conceptURIandWeights the Hashtable containing the concepts and their relative weights that compare into semantic annotations
	 * to be stored
	 * @param author the men or device that produced the annotations
	 * @param type the type of annotations
	 * @see it.polito.elite.doseWS.kernel.AnnoRepository#addAnnotations(java.lang.String, java.lang.String, java.util.Hashtable, java.lang.String, java.lang.String)
	 */
	public int addAnnotations(String fragmentURI, String fragmentXpath,
			Hashtable conceptURIandWeights, String author, String type) throws Exception
	{
		//As first, create a spectrum with provided data
		if (debug) System.out.println("DEBUG: received spectrum,deserializing");
		Spectrum initialSpectrum = new SpectrumImpl(fragmentURI,fragmentXpath,author,type,conceptURIandWeights);
		
		if (debug) System.out.println("DEBUG: deserialized spectrum:"+initialSpectrum.toString());
		
		//Expand the spectrum
		Spectrum expandedSpectrum = this.sendExpansionRequest(initialSpectrum);
		
		if (debug) System.out.println("DEBUG: received expanded spectrum:"+expandedSpectrum.toString());
		
		//tries to store the initial spectrum
		int errorCode = storage.insertSpectrum(initialSpectrum);
		if (debug) System.out.println("DEBUG: sent store request, errorCode:"+errorCode);
		
		//checking the error code
		if (errorCode < 0)
		{
			throw new Exception(((DBWrapperImpl)storage).dbStatus());
		}
		
		//tries to store the expanded spectrum
		errorCode = storage.insertXSpectrum(expandedSpectrum);
		//checking the error code
		if (errorCode < 0)
		{
			throw new Exception(((DBWrapperImpl)storage).dbStatus());
		}
		
		//if comes here it is all ok
		return 1;
	}

	/**
	 * Add a single semantic annotation to the annotation repository
	 * 
	 * @param fragmentURI the URI of the document for which the annotation should be added in the repository
	 * @param fragmentXPath the XPath of the specific substructure of the document to which the annotation is referred
	 * @param conceptURI the URI of the concept to which the annotation is referred
	 * @param weight the weight value associated to the annotation
	 * @param author the men or device that produced the annotation
	 * @param type the type of the annotation
	 * @see it.polito.elite.doseWS.kernel.AnnoRepository#addAnnotation(java.lang.String, java.lang.String, java.lang.String, double, java.lang.String, java.lang.String)
	 */
	public int addAnnotation(String fragmentURI, String fragmentXpath,
			String conceptURI, double weight, String author, String type) throws Exception 
	{
		Hashtable conceptURIandWeight = new Hashtable();
		conceptURIandWeight.put(conceptURI,new Double(weight));
		return addAnnotations(fragmentURI, fragmentXpath,conceptURIandWeight, author, type);
	}

	/* (non-Javadoc)
	 * @see it.polito.elite.doseWS.kernel.AnnoRepository#move(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	/*public int move(String originURI, String destinationURI, String author,
			String type) 
	{
		storage.
		return 0;
	}*/

	/* (non-Javadoc)
	 * @see it.polito.elite.doseWS.kernel.AnnoRepository#searchByTopic(java.util.Vector, java.util.Vector, java.lang.String)
	 */
	public Vector searchByTopic(Vector topicURIs, Vector authors, String type, double strict) {
		return storage.searchXByTopic(topicURIs,(int)(topicURIs.size()* strict));
	}
	
	private Spectrum sendExpansionRequest(Spectrum spectrum)throws Exception
	{
		//the placeholder for the expanded spectrum
		String serializedXSpectrum = "";
		//serializing the spectrum for trasmission over SOAP
		String serializedSpectrum = ((SpectrumImpl) spectrum).serialize();
		
		if(debug) System.out.println("DEBUG: trying to create communication stub");
		
		
		
		//trying to invoke the Expander service
		try
		{
//			 creates a new QName identifier for the expansion service "http://localhost:8080/"
			QName op =new QName(AnnoRepositoryImpl.server,"expandResourceSpectrum"); 
			//creating service factory
			ServiceFactory factory  = ServiceFactory.newInstance();
			//defining the service
			Service service = factory.createService(new QName("ExpanderWSService"));
			//defining the service port
			QName port = new QName("ExpanderWS");
			
			//creating the call object
			Call call = service.createCall(port);
			
			//creating the stub for the ExpanderWS
			//Service service = new Service(op);
			if(debug) System.out.println("DEBUG: created communication stub");
			//creates a web service call
			//Call call= (Call) service.createCall();
			//sets the endpoint for the call "http://localhost:8080/axis/ExpanderWS.jws"
			call.setTargetEndpointAddress(AnnoRepositoryImpl.endpoint);
			
			//sets the operation to be required by the call
			call.setOperationName(op);
			//creates the placeholder for remote method params
			Object params[] = new Object[1];
			//adds the required parameter: a serialized spectrum
			params[0]= (Object)serializedSpectrum;
			//invokes the remote service
			if(debug) System.out.println("DEBUG: invoking the expander service");
			serializedXSpectrum = (String) call.invoke(params);
		}
		catch (Exception e)
		{
			String error;
			e.printStackTrace();
			if(e instanceof RemoteException)
			{
				error = "Error invoking the expander web service: ";
				error += e;
				throw new Exception (error);
			}
			if(e instanceof ServiceException)
			{
				error = "Error creating the call to the expander web service: ";
				error += e;
				throw new Exception (error);
			}
			if(e instanceof MalformedURLException)
			{
				error = "Error on the Expander end point: ";
				error += e;
				throw new Exception (error);
			}
		}
		//deserialize the expanded spectrum
		Spectrum expandedSpectrum = (Spectrum)SpectrumImpl.deSerialize(serializedXSpectrum);
		
		return expandedSpectrum;
	}
	
	private static void loadConf(Properties config, String confFile) throws Exception
	{
		InputStream in = Expander.class.getClassLoader()
		.getResourceAsStream(confFile);
		config.load(in);
	}

	/* (non-Javadoc)
	 * @see it.polito.elite.doseWS.kernel.AnnoRepository#reverseSearch(java.lang.String, java.lang.String)
	 */
	public Vector reverseSearch(String fragmentURI, String fragmentXPath) 
	{
		//retrieve the spectrum of the given fragment
		Spectrum spectrum = AnnoRepositoryImpl.storage.getXSpectrum(fragmentURI,fragmentXPath,"default","default");
		//extract the hashtable of concepts and weights
		Hashtable conceptsAndWeights = spectrum.getConcepts();
		//gets the involved topics
		Enumeration keys = conceptsAndWeights.keys();
		//prepare a vector for transferring topics 
		Vector topics = new Vector();
		while (keys.hasMoreElements())
		{
			//add spectrum topics to the vector
			topics.add((String)keys.nextElement());
		}
		//return search results 
		return AnnoRepositoryImpl.storage.searchXByTopic(topics,1);
	}
	
	public Spectrum getSpectrum(String fragmentURI,String fragmentXPath)
	{
		//retrieve the spectrum of the given fragment
		return AnnoRepositoryImpl.storage.getXSpectrum(fragmentURI, fragmentXPath,"default","default");
	}
}
