/*
 * ExpanderImpl.java
 *
 * Created on July 6, 2004, 2:37 PM
 */

package it.polito.elite.doseWS.kernel;

//DOSE imports
import it.polito.elite.doseWS.include.Spectrum;
import it.polito.elite.doseWS.include.SpectrumImpl;
import it.polito.elite.doseWS.wrappers.Ontology;
import it.polito.elite.doseWS.wrappers.OntologyImpl;

//JAVA imports
import java.util.*;

//JGRAPHT imports
import org._3pq.jgrapht.Edge;
import org._3pq.jgrapht.alg.DijkstraShortestPath;
import org._3pq.jgrapht.edge.EdgeFactories.UndirectedWeightedEdgeFactory;
import org._3pq.jgrapht.graph.SimpleWeightedGraph;

/**
 *
 * @author  bonino
 */
public class ExpanderImpl implements Expander
{
	//unitary expanded spectra for each ontology concept
	private static Hashtable baseSpectraCache = new Hashtable();
	
	//Weight for the isA relationship
	private static double isAWeight = 0.5;
	
	//The ontology filename
	private static String ontology = "files/ontology/ontology.owl";
	
	//The ontology location type
	private static boolean useURI = false;
	
	private static boolean ready = false;
	
	//default config file name
	private final static String defaultConfigFilename = "files/config/config.conf";
	
	/** Creates a new instance of ExpanderImpl */
	public ExpanderImpl()
	{
		//void method for web service deployment using Apache Axis
	}
	
	public static void initExpander()throws Exception
	{
		ExpanderImpl.initExpander(new Double(isAWeight), ExpanderImpl.ontology, new Boolean(ExpanderImpl.useURI));
	}
	
	public static void initExpander (Double isAWeight, String ontologyFileName, Boolean isURI)throws Exception
	{
		ExpanderImpl.isAWeight=isAWeight.doubleValue();
		ExpanderImpl.ontology = ontologyFileName;
		ExpanderImpl.useURI = isURI.booleanValue();
		ExpanderImpl.initExpanderCommon();
	}
	
	public static void initExpanderCommon()throws Exception
	{
		//initialize the static variables of the Expander
		Properties config = new Properties();
		Ontology ontology = new OntologyImpl(ExpanderImpl.ontology, ExpanderImpl.useURI);
		SimpleWeightedGraph ontologyGraph = new SimpleWeightedGraph();
		//initialize the expansion graph
		ExpanderImpl.initGraph(ontology,ontologyGraph);
		//initialize the hierarchy relations in the graph
		ExpanderImpl.initHierarchy(ontology,ontologyGraph,ExpanderImpl.isAWeight);
		//initializing the other relations
		ExpanderImpl.initRelations(ontology,ontologyGraph,ExpanderImpl.isAWeight);
		//initialize the expansion cache
		ExpanderImpl.initSpectraCache(ontology,ontologyGraph,ExpanderImpl.isAWeight);
		//if all is right up to here the expander has been correctly initialized
		ExpanderImpl.ready=true;
	}
	
	public static boolean isReady()
	{
		return ExpanderImpl.ready;
	}
	
	private static void initGraph(Ontology onto, SimpleWeightedGraph graph)
	{
		//as First get all ontology concepts
		Iterator allClasses = onto.getAllClasses().iterator();
		
		//loads all graph vertices
		while(allClasses.hasNext())
		{
			//add the ontology concepts to the graph
			graph.addVertex((String)allClasses.next());
		}
		
	}
	
	private static void initHierarchy(Ontology onto, SimpleWeightedGraph graph, double isAWeight) throws Exception
	{
		try
		{
			//inits the ontology graph adding hierarchy relations between concepts
			
			//get all ontology concepts
			Set ontoNodes = graph.vertexSet();
			Iterator ontoNodesIterator = ontoNodes.iterator();
			
			//initialize the edge factory for adding edges to the ontology graph
			UndirectedWeightedEdgeFactory factory = new UndirectedWeightedEdgeFactory();
			
			//add hierarchy relationships
			while(ontoNodesIterator.hasNext())
			{
				//first concept
				String originConcept = (String) ontoNodesIterator.next();
				
				//System.err.println("Original concept:"+originConcept);
				
				//get all subclasses of the origin topic
				Iterator subClasses = onto.getSubClasses(originConcept);
				
				//add isA edges
				while(subClasses.hasNext())
				{
					//isA possess the maximum weight,
					//edges weight are defined as exp(relWeight), in that way propagation can
					//be computed as the log of the sum of edges weights between two
					//concepts
					
					//creating edge
					Edge isA = factory.createEdge(originConcept, (String)subClasses.next());
					//assigning edge value
					isA.setWeight(1.0/isAWeight);
					//adding edge to graph
					graph.addEdge(isA);
					//System.err.println("Added edge:"+isA.toString());
				}
			}
		}
		catch(Exception e)
		{
			throw new Exception("Error on expander isA init");
		}
		
		
	}
	
	private static void initRelations(Ontology onto, SimpleWeightedGraph graph,double isAWeight) throws Exception
	{
		try
		{
			//get all Relations
			Hashtable relations = onto.getAllRelations();
			
			//get all ontology classes
			Set ontoNodes = graph.vertexSet();
			Iterator ontoNodesIterator = ontoNodes.iterator();
			
			//create edge factory
			UndirectedWeightedEdgeFactory factory = new UndirectedWeightedEdgeFactory();
			
			//for each ontology concept insert edges into the graph
			while (ontoNodesIterator.hasNext())
			{
				//the starting concept
				String originConcept = (String) ontoNodesIterator.next();
				
				//enumeration of relation URIs
				Enumeration relationsEnum = relations.keys();
				
				//for each relation add linked concepts to the graph
				while(relationsEnum.hasMoreElements())
				{
					//get the related concepts
					
					//the relation URI
					String relation = (String) relationsEnum.nextElement();
					
					//the relative relation weight
					double relationWeight = ((Double)relations.get(relation)).doubleValue();
					
					//the absolute relation weight
					relationWeight*=isAWeight;
					
					//the edge weight
					double edgeWeight = (1.0/relationWeight);
					
					//get the relation ends
					Iterator endNodes = onto.getRelationEnds(originConcept, relation).iterator();
					
					//for each relation add edges into the ontology graph
					while(endNodes.hasNext())
					{
						Edge edge = factory.createEdge(originConcept, endNodes.next());
						edge.setWeight(edgeWeight);
						graph.addEdge(edge);
					}
					
				}
				
			}
							
			
		}
		catch(Exception e)
		{
			throw new Exception ("Error on expander relation init"+e);
		}
	}
	
	private static void initSpectraCache(Ontology onto, SimpleWeightedGraph graph,double isAWeight)
	{
		//get all ontology classes
		Set ontoNodes = graph.vertexSet();
		Iterator ontoNodesIterator = ontoNodes.iterator();
		
		//for each node extract the corresponding base spectrum
		while (ontoNodesIterator.hasNext())
		{
			String conceptSeed = (String) ontoNodesIterator.next();
			Hashtable baseSpectrum = ExpanderImpl.extractBaseSpectrum(conceptSeed, graph);
			ExpanderImpl.baseSpectraCache.put(conceptSeed,baseSpectrum);
		}
		
	}
	
	private static Hashtable extractBaseSpectrum(String concept,SimpleWeightedGraph graph)
	{
		//extracts the base spectrum for a given concept
		Hashtable baseSpectrum = new Hashtable();
		
		//get all ontology classes
		Set ontoNodes = graph.vertexSet();
		Iterator ontoNodesIterator = ontoNodes.iterator();
		
		//extract the minimum path between "concept" and all other ontology concept
		while(ontoNodesIterator.hasNext())
		{
			String node = (String) ontoNodesIterator.next();
			double expansionFactor = 0.0;
			if(!node.equals(concept))
			{
				//extract the shortest path between concept and node
				List edgeList = DijkstraShortestPath.findPathBetween(graph,concept,node);
				
				//if exist a path between concept and node evaluate the weight of such path
				if((edgeList!=null)&&(!edgeList.isEmpty()))
				{
					Iterator edgeIterator = edgeList.iterator();
					expansionFactor = 1.0;
					while(edgeIterator.hasNext())
						expansionFactor*=1.0/((Edge)edgeIterator.next()).getWeight();	
				}
			}
			else
			{
				expansionFactor = 1.0;
			}
			
			//if the expansion Factor is not null, i.e., there is some relation between
			//concept and node add the component to the base Specrum
			if(expansionFactor != 0.0)
				baseSpectrum.put(node,new Double(expansionFactor));
			
		}
		return baseSpectrum;
	}
	
	private Hashtable expandSingleSpectrum(String concept, Double weight)
	{
		//get the base spectrum and multiply each entry by weight
		
		//the expanded spectrum
		Hashtable expandedSpectrum = new Hashtable();
		
		//gets the base spectrum
		Hashtable baseSpectrum = (Hashtable) ExpanderImpl.baseSpectraCache.get(concept);
		//gets all components of the base spectrum
		Enumeration baseSpectrumEnum = baseSpectrum.keys();
		
		//evaluates components of the expanded spectrum
		while (baseSpectrumEnum.hasMoreElements())
		{
			//single component of the spectrum
			String topic = (String) baseSpectrumEnum.nextElement();
			
			//base value
			double baseWeight = ((Double)baseSpectrum.get(topic)).doubleValue();
			
			//expanded value
			expandedSpectrum.put(topic,new Double(baseWeight*weight.doubleValue()));
		}
		
		return expandedSpectrum;
	}
	
	private Hashtable sumSpectra(Hashtable sa, Hashtable sb)
	{
		//sums two hashtables representing conceptual spectra
				
		//the resulting Hashtable
		Hashtable sum = new Hashtable();
		//creates a common topic set with no duplicates
		HashSet topics = new HashSet(sa.keySet());
		topics.addAll(sb.keySet());
		
		//compute the sum
		Iterator topicsIter = topics.iterator();
		
		while(topicsIter.hasNext())
		{
			double a=0.0,b=0.0;
			//the single spectra component
			String topic = (String)topicsIter.next();
			
			//the component value in sa
			Double weight = (Double)sa.get(topic);
			if(weight!=null) a=weight.doubleValue();
			
			//the component value in sb
			weight = (Double)sb.get(topic);
			if(weight!=null) b=weight.doubleValue();
			
			//creating the sum
			sum.put(topic,new Double(a+b));
			
		}
		
		return sum;
	}
	public Spectrum expandResourceSpectrum(Spectrum rawSpectrum)
	{
		//the expanded innerSpectrum
		Hashtable expandedSpectrum = new Hashtable();
		//extract the innerSpectrum representation
		Hashtable innerSpectrum = rawSpectrum.getConcepts();
		
		//expand each component and sum contributes
		
		//get the starting components
		Enumeration components = innerSpectrum.keys();
		
		while(components.hasMoreElements())
		{
			String topic = (String) components.nextElement();
			Hashtable singleXSpectrum = this.expandSingleSpectrum(topic, (Double) innerSpectrum.get(topic));
			expandedSpectrum = this.sumSpectra(expandedSpectrum, singleXSpectrum);
		}
			
		Spectrum xSpectrum = new SpectrumImpl(rawSpectrum.getFragmentURI(),rawSpectrum.getFragmentXPath(),rawSpectrum.getAuthor(),rawSpectrum.getType(),expandedSpectrum);
		return xSpectrum;
	}
	
}
