/*
 *DBWrapper.java
 *
 *Created on July 5, 2004, 5:42 PM
 *@author Michele - ita
 */
package it.polito.elite.doseWS.wrappers;
import java.sql.*;
import java.util.*;
import java.util.Vector;
import it.polito.elite.doseWS.include.*;

/**
 *
 * This class implements DBWrapper
 *
 *<br>
 * configuration parameters in config.conf
 *<pre>
 * dbDriver=COM.cloudscape.core.RmiJdbcDriver
 * dbURI=jdbc:cloudscape:rmi:jGuru
 * dbUser=sa
 * dbPassword=admin
 *</pre>
 */
/* DBMS structure (for PostgreSQL):
 
 -- -- -- -- -- -- -- -- -- -- --
CREATE SEQUENCE wsfragment_idwsfragment_seq,
--
CREATE TABLE wsfragment (
  idwsfragment INTEGER DEFAULT nextval('wsfragment_idwsfragment_seq') NOT NULL,
  uri          VARCHAR (2047) NOT NULL,
  xpath        VARCHAR (2047) NOT NULL,
  CONSTRAINT   wsfragment_pkey PRIMARY KEY (idwsfragment),
  CONSTRAINT   wsfragment_chk  UNIQUE (uri,xpath)
);
--
CREATE SEQUENCE wsauthor_idwsauthor_seq;
--
CREATE TABLE wsauthor (
  idwsauthor INTEGER DEFAULT nextval('wsauthor_idwsauthor_seq') NOT NULL,
  name       VARCHAR (127) NOT NULL,
  CONSTRAINT wsauthor_pkey PRIMARY KEY (idwsauthor),
  CONSTRAINT wsauthor_chk  UNIQUE (name)
);
--
CREATE SEQUENCE wsanntype_idwsanntype_seq;
--
CREATE TABLE wsanntype (
  idwsanntype INTEGER DEFAULT nextval('wsanntype_idwsanntype_seq') NOT NULL,
  tname       VARCHAR (127) NOT NULL,
  CONSTRAINT  wsanntype_pkey PRIMARY KEY (idwsanntype),
  CONSTRAINT  wsanntype_chk  UNIQUE (tname)
);
--
CREATE SEQUENCE wstopic_idwstopic_seq;
--
CREATE TABLE wstopic (
  idwstopic  INTEGER DEFAULT nextval('wstopic_idwstopic_seq') NOT NULL,
  tname      VARCHAR (127) NOT NULL,
  CONSTRAINT wstopic_pkey PRIMARY KEY (idwstopic),
  CONSTRAINT wstopic_chk  UNIQUE (tname)
);
--
CREATE TABLE wsannotations (
  idwsfragment INTEGER NOT NULL,
  idwstopic    INTEGER NOT NULL,
  idwsauthor   INTEGER NOT NULL,
  idwstype     INTEGER NOT NULL,
  weight       DOUBLE PRECISION,
  date         TIMESTAMP,
  CONSTRAINT   wsannotations_chk
               UNIQUE (idwsfragment,idwstopic,idwsauthor,idwstype)
);
--
CREATE TABLE wsviewannotations (
  idwsfragment INTEGER NOT NULL,
  idwstopic    INTEGER NOT NULL,
  idwsauthor   INTEGER NOT NULL,
  idwstype     INTEGER NOT NULL,
  weight       DOUBLE PRECISION,
  date         TIMESTAMP,
  CONSTRAINT   wsviewannotations_chk
               UNIQUE (idwsfragment, idwstopic,idwsauthor,idwstype)
);
--
-- annotations' referential integrity
--
ALTER TABLE wsannotations
  ADD CONSTRAINT wsfragment_ann_ref
                 FOREIGN KEY (idwsfragment)
                 REFERENCES wsfragment(idwsfragment)
  ON DELETE CASCADE;
--
ALTER TABLE wsannotations
  ADD CONSTRAINT wstopic_ann_ref
                 FOREIGN KEY (idwstopic)
                 REFERENCES wstopic(idwstopic)
  ON DELETE CASCADE;
--
ALTER TABLE wsannotations
  ADD CONSTRAINT wsauthor_ann_ref
                 FOREIGN KEY (idwsauthor)
                 REFERENCES wsauthor(idwsauthor)
  ON DELETE CASCADE;
--
ALTER TABLE wsannotations
  ADD CONSTRAINT wstype_ann_ref
                 FOREIGN KEY (idwstype)
                 REFERENCES wsanntype(idwsanntype)
  ON DELETE CASCADE;
--
-- expanded annotations' referential integrity
--
ALTER TABLE wsviewannotations
  ADD CONSTRAINT wsfragment_viewann_ref
                 FOREIGN KEY (idwsfragment)
                 REFERENCES wsfragment(idwsfragment)
  ON DELETE CASCADE;
--
ALTER TABLE wsviewannotations
  ADD CONSTRAINT wstopic_viewann_ref
                 FOREIGN KEY (idwstopic)
                 REFERENCES wstopic(idwstopic)
  ON DELETE CASCADE;
--
ALTER TABLE wsviewannotations
  ADD CONSTRAINT wsauthor_viewann_ref
                 FOREIGN KEY (idwsauthor)
                 REFERENCES wsauthor(idwsauthor)
  ON DELETE CASCADE;
--
ALTER TABLE wsviewannotations
  ADD CONSTRAINT wstype_viewann_ref
                 FOREIGN KEY (idwstype)
                 REFERENCES wsanntype(idwsanntype)
  ON DELETE CASCADE;
--
--
-- synset tables
--
CREATE SEQUENCE wssynset_idsynset_seq;
--
CREATE TABLE wssynset (
  idwssynset INTEGER DEFAULT nextval('wssynset_idsynset_seq') NOT NULL,
  idwstopic   INTEGER NOT NULL,
  lang        CHAR(2),
  content     VARCHAR(255),
  CONSTRAINT  wssynset_pkey PRIMARY KEY (idwssynset),
  CONSTRAINT  wssynset_lock UNIQUE (idwstopic,lang)
);
 
ALTER TABLE wssynset
  ADD CONSTRAINT wssynset_viewtopic_ref
                 FOREIGN KEY (idwstopic)
                 REFERENCES wstopic(idwstopic)
  ON DELETE CASCADE;
 
CREATE SEQUENCE wsmword_idwsmword_seq;
 
CREATE TABLE wsmword (
  idwsmword  INTEGER DEFAULT nextval('wsmword_idwsmword_seq') NOT NULL,
  content    VARCHAR(255),
  CONSTRAINT wsmword_pkey PRIMARY KEY (idwsmword)
);
--
CREATE TABLE wsoccour (
  idwssynset INTEGER NOT NULL,
  idwsmword  INTEGER NOT NULL,
  occourence INTEGER DEFAULT 0,
  weight     DOUBLE PRECISION DEFAULT 0,
  CONSTRAINT wsoccour_pkey
             PRIMARY KEY (idwssynset,idwsmword)
);
 
ALTER TABLE wsoccour
  ADD CONSTRAINT wsoccour_idwssynset_ref
                 FOREIGN KEY (idwssynset)
                 REFERENCES wssynset(idwssynset)
  ON DELETE CASCADE;
 
ALTER TABLE wsoccour
  ADD CONSTRAINT wsoccour_idwsmword_ref
                 FOREIGN KEY (idwsmword)
                 REFERENCES wsmword(idwsmword)
  ON DELETE CASCADE;
 
--
 
 */
public class DBWrapperImpl implements DBWrapper {
    /**
     * SQL-Level error (ie. syntax errors in SQL, bad parameters...)
     */
    public static int EBADSQL = -1;
    /**
     * Required key not found
     */
    public static int ENOENT = -2;
    /**
     * Operation failed for external causes
     */
    public static int EAGAIN = -3;
    /**
     * Wrong parameter
     */
    public static int EPARAM = -4;
    /**
     * set to true to see debug messages in stdout
     */
    private static boolean DEBUG = true;
        /**
     * set to true to see debug messages in stdout
     */
    private boolean checkann = false;
    /**
     * DBMS connection
     */
    private Connection con;
    /**
     * Statement Object
     */
    private Statement stmt;
    /**
     * jdbc parameters
     */
    private String dbdriver;
    /**
     * jdbc parameters
     */
    private String dburi;
    /**
     * jdbc parameters
     */
    private String dbuser;
    /**
     * jdbc parameters
     */
    private String dbpassword;
    /**
     * last SQL error made
     */
    private String lasterror = "";
    /**
     * Configuration storage
     */
    private static Properties config;
    
    
    
    /**
     *  An Implementation of DBWrapper
     *
     * @param dbDriver JDBC driver to use
     * @param dbURI DBMS URI to use
     * @param dbUser DBMS user
     * @param dbPassword DBMS password
     * @throws Exception in error
     */
    public DBWrapperImpl(String dbDriver,String dbURI, String dbUser, String dbPassword) throws Exception {
        
        dbdriver = dbDriver;
        dburi = dbURI;
        dbuser = dbUser;
        dbpassword = dbPassword;
        try   {
            /* loading jdbc w/ newInstance */
            Class.forName( dbdriver ).newInstance();
        }
        catch( Exception e )  {
            perror( "DBWrapperImpl:" +e.getMessage()+";"+
            "dbdriver="+dbdriver+";dburi="+dburi+";dbuser=" +
            dbuser+";dbpassword="+dbpassword);
            throw new Exception(this.lasterror);
        };
        try {
            con = DriverManager.getConnection( dburi,dbuser,dbpassword);
            DatabaseMetaData dbmd = con.getMetaData();
            if (DEBUG)
                System.out.println("DBMS:\t\t" + dbmd.getDatabaseProductName()  +
                "\n\tRelease:" + dbmd.getDatabaseProductVersion() +
                "\n\tDriver: " + dbmd.getDriverName()  +
                "\n\tVersion:"+ dbmd.getDriverVersion());
            stmt = con.createStatement();
        }
        catch (Exception SQLe) {
            perror( "DBWrapperImpl/connection:" + dburi + ";" +
            SQLe.getMessage() );
            if( con != null) {
                try { con.close(); }
                catch( Exception e ) {}
            }
            throw new Exception(this.lasterror);
        } /* catch */
    } /* DBWrapper */
    /**
     * Set the checkann flag. If set to true, insering nuplicated 
     * annotation values will succeed (oder weights will be mantained)
     * else will be generated an exception
     *
     *  This behaviour is valid only if there is a RDBMS under
     *
     * @param b the checkann value
     */
    public void setCheckann (boolean b) {
        checkann = b;
    }
    /**
     * reads thre checkann boolean value
     *@return the value of checkann
     *
     */
    public boolean getCheckann () {
        return checkann;
    }
    
    /**
     * Insert a block of annotations for a Spectrum
     * @return <0 in error, otherwise the number of annotations
     *
     * @param spectrum The specrum to be inserted
     * @throws Exception for serious errors
     */
    public int insertSpectrum(Spectrum spectrum) throws Exception{
        if(DEBUG)
            System.out.println("DEBUG-DBWrapperImpl: received spectrum" +
            spectrum.toString());
        return insertSpectrumCommon("wsannotations",spectrum);
    }
    
    /**
     * Insert a block of annotations for an expanded Spectrum
     * @return <0 in error, otherwise the number of annotations
     *
     * @param spectrum The specrum to be inserted
     * @throws Exception for serious errors
     */
    
    public int insertXSpectrum(Spectrum spectrum) throws Exception {
        return insertSpectrumCommon("wsviewannotations",spectrum);
    }
    /**
     * Insert a block of annotations for a Spectrum, common code
     * @return <0 in error, otherwise the number of annotations
     * @param table the database table to use
     * @param spectrum The specrum to be inserted
     * @throws Exception for serious errors
     */
    
    private int insertSpectrumCommon(String table, Spectrum spectrum)
    throws Exception {
        ResultSet rs;
        PreparedStatement sqlprep = null;
        PreparedStatement sqlprep1 = null;
        String sql = null;
        String name = null;
        Double value = null;
        String columns[] = {"uri","xpath"};
        String values[] = new String[2];
        // boolean bError = false;
        int fragID = 0, xpathID = 0, authorID = 0, typeID = 0, i = 0;
        lasterror="";
        Integer frag;
        con.setAutoCommit(false);
        values[0]=spectrum.getFragmentURI();
        values[1]=spectrum.getFragmentXPath();
        
        try {
            frag=selectGeneric("wsfragment", "idwsfragment", columns, values);
            if (frag != null) {
                fragID = frag.intValue(); /* the fragment exists */
                if (DEBUG) {
                    System.out.println("#InsertSpectrumCommon: old fragment #"+fragID);
                }
            }
            else {
                frag=insertGeneric("wsfragment", "idwsfragment",
                "wsfragment_idwsfragment_seq", columns, values);
                if (frag == null) {
                    con.rollback();
                    perror("InsertSpectrumCommon: Cannot insert a new fragment!");
                    con.setAutoCommit(false);
                    return EAGAIN;
                }
                fragID=frag.intValue();
                if (DEBUG) {
                    System.out.println("#InsertSpectrumCommon: fragment #"+fragID);
                }
            }
            
            sql="SELECT idwsauthor FROM wsauthor " +
            "WHERE name = '" + spectrum.getAuthor() + "' ";
            rs = stmt.executeQuery(sql);
            if (rs.next()) {
                authorID = rs.getInt(1);/* author check */
            }
            else {
                authorID=0; /* if the author doesn't exist, use a default value */
            }
            
            sql="SELECT idwsanntype FROM wsanntype " +
            "WHERE tname = '" + spectrum.getType() + "' ";
            rs = stmt.executeQuery(sql);
            if (rs.next()) {
                typeID = rs.getInt(1);/* type check */
            }
            else {
                typeID=0; /* if the type doesn'exists, use a default value */
            }
            /* now we can insert the data into the correct table */
            sql = "INSERT INTO " + table +
            " (idwsfragment, idwsauthor, idwstype, date, idwstopic, weight) " +
            "   SELECT " + fragID + "," + authorID + "," + typeID +
            "," + " CURRENT_TIMESTAMP " + " , f.idwstopic , ? " +
            "   FROM wstopic f" +
            "   where f.tname = ?";
            sqlprep = con.prepareStatement(sql);
            if (checkann) {
                sql=" SELECT count(*) from "+ table +" a, wstopic f " +
                " WHERE a.idwsfragment ="+ fragID +
                " AND   a.idwsauthor = "+ authorID +
                " AND   a.idwstopic = f.idwstopic " +
                " AND   f.tname = ? ";
                sqlprep1 = con.prepareStatement(sql);
            }
            Hashtable h = spectrum.getConcepts();
            for (Enumeration en=h.keys(); en.hasMoreElements();i++ ) {
                name = (String)en.nextElement();
                value = (Double)h.get(name);
                if (DEBUG)
                    System.out.println("#InsertSpectrumCommon:(name:"+name +")(value:" +value+")");
                if (checkann) {
                    sqlprep1.setString(1,name);
                    rs = sqlprep1.executeQuery();
                    if (!rs.next()) {
                        if (DEBUG) {
                         System.out.println("*InsertSpectrumCommon: row must exists bu still not found... giving up"+name);
                        }
                        return (EAGAIN);
                    }
                    if (rs.getInt(1)>0) {
                        if (DEBUG) {
                            System.out.println("##InsertSpectrumCommon: duplicated annotation "+name);
                        }
                        continue;
                    }
                }
                sqlprep.setString(2,name);
                sqlprep.setDouble(1,value.doubleValue());
                
                sqlprep.executeUpdate();
                
                
            }
            
        }
        catch (Exception SQLe) {
            lasterror = "InsertSpectrumCommon:"+SQLe.getMessage() +
            "\n(sql:"+sql+")"+
            "\n(value:"+value+")(name:"+name+")";
            perror(lasterror);
            con.rollback();
            con.setAutoCommit(true);
            return (EBADSQL);
        }
        con.commit();
        con.setAutoCommit(true);
        return i;
    }
    /**
     * gives back the block of annotations, given fragment
     * (author and type are optional)
     *@return the requested Spectrum
     *@param uri fragment's URI
     *@param xPath fragment's xPath
     *@param author fragment's author
     *@param type e fragment's type
     */
    
    public Spectrum getSpectrum(String uri, String xPath, String author,
    String type) {
        return selectSpectrum("wsannotations",uri,xPath,author,type);
    }
    /**
     * gives back the block of refined annotations, given fragment
     * (author and type are optional)
     *@return the requested Spectrum
     *@param uri fragment's URI
     *@param xPath fragment's xPath
     *@param author fragment's author
     *@param type e fragment's type
     */
    public Spectrum getXSpectrum(String uri, String xPath, String author,
    String type) {
        return selectSpectrum("wsviewannotations",uri,xPath,author,type);
    }
    /**
     * gives back the block of  annotations, given fragment
     * (author and type are optional) - common code
     *@return the requested Spectrum
     *@param uri fragment's URI
     *@param xPath fragment's xPath
     *@param author fragment's author
     *@param type  fragment's type
     *@param table table to operate
     */
    
    private Spectrum selectSpectrum(String table, String uri, String xPath,
    String author, String type) {
        String sql="", name;
        Double value;
        ResultSet rs;
        boolean bError = false;
        lasterror="";
        try {
            sql = "SELECT ann.weight,top.tname FROM "+ table +
            " ann, wstopic top, wsfragment frag " +
            (chkNotNull(author) ? ", wsauthor auth " : " ") +
            (chkNotNull(type)   ? ", wsanntype typ " : " ") +
            " WHERE ann.idwsfragment = frag.idwsfragment " +
            " AND   frag.uri = '" +    uri + "' " +
            " AND   frag.xpath = '" + xPath  +"' " +
			" AND   top.idwstopic = ann.idwstopic " +
            (chkNotNull(author) ? " AND auth.idwsauthor = ann.idwsauthor AND auth.name='" + author + "' " :  " ") +
            (chkNotNull(type) ? " AND typ.idwsanntype = ann.idwstype AND typ.tname='" + type + "' " :  " ");
            rs = stmt.executeQuery(sql);
            System.out.println("DBWRAPPERIMPL-->"+sql);
            Hashtable H = new Hashtable(20);
            int i = 0;
            while (rs.next()) { /*fragment check */
                i++;
                name = rs.getString(2);
                value = new Double(rs.getDouble(1));
                H.put(name ,value);
            };
            if (i == 0)
                return null;
            Spectrum S = new SpectrumImpl(uri,xPath, author, type, H);
            return S;
        }
        catch (Exception SQLe) {
            lasterror = "SelectSpectrum:"+SQLe.getMessage() +"(sql:"+sql+")";
            perror(lasterror);
            return null;
        }
    }
    /**
     * Updates a spectrum: the actual behaviour is to delete the existing specrum,
     * then inset the new specrum
     * @param spectrum the Spectrum that will be updated
     * @throws Exception In case of error.
     * @return Number of spectrum element inseted or a negative number in error
     */
    public int updateSpectrum(Spectrum spectrum)
    throws Exception {
        int dummy = deleteSpectrumCommon
        ("wsannotations", spectrum.getFragmentURI(), spectrum.getFragmentXPath(),
        spectrum.getAuthor(), spectrum.getType());
        return insertSpectrumCommon("wsannotations",spectrum);
    }
    /**
     * delete an annotations block
     * @return < 0 in case of error > 0 in succesful completion
     * @param uri annotation's URI
     * @param xPath annotation's xpath
     * @param author annotation's author
     * @param type annotation's type
     */
    public int deleteSpectrum(String uri, String xPath, String author, String type) {
        return deleteSpectrumCommon("wsannotations",uri,xPath,author,type);
    }
    /**
     * delete an expanded annotations block
     * @return < 0 in case of error > 0 in succesful completion
     * @param uri annotation's URI
     * @param xPath annotation's xpath
     * @param author annotation's author
     * @param type annotation's type
     */
    
    public int deleteXSpectrum(String uri, String xPath, String author, String type) {
        return deleteSpectrumCommon("wsviewannotations",uri,xPath,author,type);
    }
    /**
     * delete annotations, common code
     * @return < 0 in case of error > 0 in succesful completion
     * @param uri annotation's URI
     * @param xPath annotation's xpath
     * @param author annotation's author
     * @param type annotation's type
     * @param table database table to use
     */
    
    private int deleteSpectrumCommon(String table, String uri, String xPath,
    String author, String type) {
        String sql = "" , name;
        Double value;
        boolean bError = false;
        lasterror="";
        try {
            sql = "DELETE FROM "+ table +
            " WHERE idwsfragment = ( SELECT idwsfragment FROM wsfragment WHERE uri = '"+uri+"' AND xpath = '"+xPath+"' )" +
            (chkNotNull(author) ? " AND  idwsauthor = (SELECT idwsauthor FROM wsauthor WHERE name = '"+author+"')" : "\n") +
            (chkNotNull(type) ? " AND  idwstype = (SELECT idwsanntype FROM wsanntype WHERE tname = '"+type+"')" : "\n");
            return stmt.executeUpdate(sql);
        }
        catch (Exception SQLe) {
            lasterror = "DeleteSpectrumCommon:"+SQLe.getMessage() +"(sql:"+sql+")";
            perror(lasterror);
            return (EBADSQL);
        }
    }
    /**
     * last database error
     * @return A string with the description of last database error
     */
    public String dbStatus() {
        return lasterror;
    }
    /**
     * give back the spectra that are compatible with the one of the given topicURI, ie.
     * some or all of the values are present on the given topicURI spectra
     *@return Spectrum Vector of suitable data
     *@param topicURIs vector of tpocuri
     *@param level number of occoureances respect the given fragment.
     * A spectrum to be considered valid shoul have at least this number of
     *occorrences. If level=1 will make OR of the terms. if level=topicURIs.lenght()
     * will make an AND of terms
     *
     */
    public Vector searchXByTopic(Vector topicURIs, int level) {
        String sql ="";
        String name;
        ResultSet rs;
        PreparedStatement sqlprep;
        Double value;
        boolean bError = false;
        Vector iuri = new Vector();
        Vector iauth = new Vector();
        Vector itype = new Vector();
        Vector result = new Vector();
        String uri,xpath,author,type;
        Integer ii;
        int j = 0;
        lasterror="";
        try {
            /* query to extract suitable spectrums
             Using group BY and having we can simulate an ANDing and ORING
             within the given topicURI, and obrerve the behaviour of specrum searching
             with values betwwen 1 and topicuris.length.
             
             This will have to be profiled | optimized
             */
            String virg ="";
            sql = "SELECT an.idwsfragment,an.idwsauthor,an.idwstype, COUNT(an.idwstopic)" +
            " FROM wsviewannotations an, wstopic f " +
            " WHERE an.idwstopic = f.idwstopic " +
            " AND f.tname IN ( ";
            for (int i=topicURIs.size()-1;i>=0;i--) {
                name = topicURIs.get(i).toString();
                sql = sql + virg + "'" +
                /* has to be done ? */
                // name.replaceAll("'","''")
                name +
                "'";
                virg =", ";
            }
            sql = sql +" ) GROUP BY an.idwsfragment,an.idwsauthor,an.idwstype HAVING COUNT(an.idwstopic) >= " + level;
            rs = stmt.executeQuery(sql);
            while (rs.next()) {
                j++;
                iuri.add(new Integer(rs.getInt(1)));
                iauth.add(new Integer(rs.getInt(2)));
                itype.add(new Integer(rs.getInt(3)));
            };
            for (int i=iuri.size()-1;i>=0;i--) {
                sql = "SELECT ann.weight,top.tname FROM wsviewannotations ann, wstopic top " +
                " WHERE ann.idwsfragment = " + iuri.get(i) +
                " AND ann.idwsauthor = " + iauth.get(i) +
                " AND ann.idwstype = " + itype.get(i) +
                " AND ann.idwstopic = top.idwstopic ";
                rs = stmt.executeQuery(sql);
                Hashtable H = new Hashtable(20);
                int k = 0;
                while (rs.next()) {
                    /* the fragment is in the DB ? */
                    k++;
                    name = rs.getString(2);
                    value = new Double(rs.getDouble(1));
                    H.put(name ,value);
                };
                sql="SELECT uri,xpath FROM wsfragment " +
                "WHERE idwsfragment = " + iuri.get(i).toString();
                rs = stmt.executeQuery(sql);
                if (rs.next()) {
                    /* get uri and xpath */
                    uri = rs.getString(1);
                    xpath = rs.getString(2);
                }
                else {
                    uri = null;
                    xpath = null;
                }
                sql="SELECT name FROM wsauthor " +
                "WHERE idwsauthor =  "+ iauth.get(i).toString();
                rs = stmt.executeQuery(sql);
                if (rs.next()) {
                    /* get the author... */
                    author = rs.getString(1);
                }
                else {
                    author = null;
                }
                sql="SELECT tname FROM wsanntype " +
                "WHERE idwsanntype = " + itype.get(i).toString();
                rs = stmt.executeQuery(sql);
                if (rs.next()) {
                    /* ... and the type of annotation too */
                    type = rs.getString(1);
                }
                else {
                    type = null;
                }
                /* ok... we can now build a suitable Spectrum */
                result.add( new SpectrumImpl(uri,xpath,author,type, H));
            }
            if (j != 0)
                return result;
            return null;
        }
        catch (Exception SQLe) {
            // SQLe.printStackTrace();
            lasterror = "SearchXbyTopic:"+SQLe.getMessage()+"(sql:"+sql+")";
            perror(lasterror);
            return null;
        }
    }
    /**
     * Check a String if is not null or empty
     *@param S string to check
     *@return true if the string has some content, false otherwise
     */
    private static boolean chkNotNull(String S) {
        if (S == null)
            return false;
        if (S.length() <= 0)
            return false;
        return true;
    }
    /**
     * print a string on standard error...
     * DISABLE ON PRODUCTION CODE!
     *@param s error string
     */
    private static void perror(String s) {
        System.err.println(" *DBWrapper: "+s);
    }
    /**
     *seleziona un topic dal database
     *@param topic il topic da cercare (es: http://www.www#001/
     *@return null se il topic non esiste, oppure l'idtopic se esiste
     *@throws Exception In caso di errore genera una Exception
     */
    public Integer selectTopic(String topic) throws Exception {
        String v[] = {"tname"};
        String w[] = {topic};
        return selectGeneric("wstopic","idwstopic",v,w);
    }
    
    /**
     * Returns the key of the annotation type, null if not found
     * @param type string representing the annotation type
     * @throws Exception see selectGeneric
     * @return the key of the type
     */    
    public Integer selectType(String type) throws Exception {
        String v[] = {"tname"};
        String w[] = {type};
        return selectGeneric("wsanntype","idwsanntype",v,w);
    }
    /**
     * Returns the key of the annotation author, null if not found
     * @param author the author sting
     * @throws Exception see selectGeneric
     * @return the author key
     */    
    public Integer selectAuthor(String author) throws Exception {
        String v[] = {"name"};
        String w[] = {author};
        return selectGeneric("wsauthor","idwsauthor",v,w);
    }
    
    /**
     * Insert an author
     * @param author the author name
     * @throws Exception see insertGeneric
     * @return  the key of the inserted row, or null if the operation fails
     */    
    public Integer insertAuthor(String author) throws Exception {
        String column[] = new String[1];
        column[0]="name";
        String val[]= new String[1];
        val[0] = author;
        
        return insertGeneric("wsauthor","idwsauthor","wsauthor_idwsauthor_seq",column,val);
    }
    
    /**
     * Insert an annotation type
     * @param anntype the annotation type name
     * @throws Exception see insertGeneric
     * @return the key of the inserted row, or null if the operation fails
     */    
    public Integer insertType(String anntype) throws Exception {
        String column[] = new String[1];
        column[0]="tname";
        String val[]= new String[1];
        val[0] = anntype;
        
        return insertGeneric("wsanntype","idwsanntype","wsanntype_idwsanntype_seq",column,val);
    }
    
    
    /**
     *insert a topic
     *@param topic the topic to insert (es: http://www.www#001/)
     *@return null if the operation fails, the topic key otherwise
     *
     *@throws Exception see insertGeneric
     */
    public Integer insertTopic(String topic)
    throws Exception {
        String column[] = new String[1];
        column[0]="tname";
        String val[]= new String[1];
        val[0] = topic;
        
        return insertGeneric("wstopic","idwstopic","wstopic_idwstopic_seq",column,val);
    }
    /**
     * Inserts an MWord Array
     * @return null in error, otherwise the numer or new rows
     * @param M MWord array to insert
     * @throws SQLException In case of error
     */
    private Integer insertMWord(MWord[] M ) throws SQLException {
        ResultSet rs;
        PreparedStatement sqlprep;
        String parole[] = new String[M.length];
        
        /* using selectgeneric/insertgeneric will slow down things, so we''write custom SQL */
        
        con.setAutoCommit(false);
        try {
            String sql = "SELECT idwsmword FROM wsmword " +
            " WHERE content = ?";
            sqlprep = con.prepareStatement(sql);
            int contaparole=0;
            for (int i=0; i<parole.length;++i) {
                sqlprep.setString(1,M[i].getMWord());
                rs = sqlprep.executeQuery();
                if ( rs.next() ) {
                    // do nothing
                }
                else {
                    parole[contaparole++]=M[i].getMWord();
                }
                rs.close();
            };
            String sql1 = " INSERT INTO wsmword(content) values(?)";
            sqlprep = con.prepareStatement(sql1);
            for (int i=0; i<contaparole;++i) {
                sqlprep.setString(1,parole[i]);
                if (sqlprep.executeUpdate()<=0) {
                    con.rollback();
                    con.setAutoCommit(true);
                    return null;
                }
            };
            con.commit();
            //con.setAutoCommit(true);
            return new Integer(contaparole);
        }
        catch (SQLException E) {
            perror(lasterror = "insertMWord:"+E.getMessage());
            con.rollback();
            con.setAutoCommit(true);
            return null;
        }
    }
    /**
     * Inssert a row into SynsetTable
     * @param topicURI Synset's topicURI
     * @param lang synset's lagnuage (two characters)
     * @return null in error, otherwise an integer
     * representing the value of the key in the table (>0)
     * @throws Exception in error
     */
    public Integer insertSynset(String topicURI, String lang)
    throws Exception {
        String v[] = {"tname"};
        String w[] = {topicURI};
        Integer uriKey = selectGeneric("wstopic","idwstopic",v,w);
        if (uriKey == null)
            return null;
        String column[]=new String[2];
        column[0]="idwstopic";
        column[1]="lang";
        String val[]=new String[2];
        val[0]=uriKey.toString();
        val[1]=lang;
        return insertGeneric("wssynset","idwssynset","wssynset_idsynset_seq",column,val);
    }
    /**
     *Cerca una riga nella tabella sysnset
     *@param topicURI URI referenced by the synset (must exist before insering
     * the sysnset)
     *@param lang two-charachet rapresentation of the synset language
     *@return null if the operation fail, othewise the key of the row inserted
     *@throws Exception see selectGeneric
     */
    public Integer selectSynset(String topicURI, String lang)
    throws Exception {
        String v[] = {"tname"};
        String w[] = {topicURI};
        Integer uriKey = selectGeneric("wstopic","idwstopic",v,w);
        if (uriKey == null)
            return null;
        String column[]=new String[2];
        column[0]="idwstopic";
        column[1]="lang";
        String val[]=new String[2];
        val[0]=uriKey.toString();
        val[1]=lang;
        return selectGeneric("wssynset","idwssynset",column,val);
    }
    /**
     * Elimina un Synset, dato topicUri e linguaggio nella tabella sysnset
     * @return null in caso di errore altrimenti il valore della chiave in caso di successo
     * @param topicURI l'uri a cui si riferisce il synset
     * @param lang lingua del synset
     * @throws Exception si` spara un'eccezione
     */
    public Integer deleteSynset(String topicURI, String lang) throws Exception{
        Integer uriKey = selectGeneric("wstopic","idwstopic",new String[]{"tname"},new String[]{topicURI});
        if (uriKey == null)
            return null;
        String column[]={"idwstopic","lang"};
        String val[]={uriKey.toString(),lang};
        return deleteGenericValue("wssynset",column,val);
    }
    /**
     * Elimina un Synset, dato topicUri e linguaggio nella tabella sysnset
     * @return null in caso di errore altrimenti il valore della chiave in caso di successo
     * @param i chiave da eliminare
     * @throws Exception eccezione in caso di errore
     */
    public Integer deleteSynset(Integer i) throws Exception {
        return deleteGenericKey("wssynset","idwssynset",i);
    }
    /**
     * elimina un synset, data la chiave
     * @return null in caso di errore
     * @param S da eliminare (ATTENZIONE: non elimina solo quello che
     *         si trova nelle occour ....)
     * @throws Exception In caso di problemi al DB spara un'eccezione
     */
    public Integer deleteSynset(Synset S) throws Exception {
        String topicURI = S.getConcept();
        String lang = S.getLanguage();
        // MWord mword[]=S.getMWords();
        try {
            PreparedStatement sqlprep;
            Integer i;
            String sql = "DELETE FROM wssynset "+
            " WHERE  lang = ? " +
            " AND idwstopic = (SELECT idwstopic FROM wstopic WHERE tname = ?) " ;
            sqlprep = con.prepareStatement(sql);
            sqlprep.setString(1,lang);
            sqlprep.setString(2,topicURI);
            i = new Integer(sqlprep.executeUpdate());
            return i;
        }
        catch (SQLException SQ) {
            lasterror = "DeleteSynset:"+SQ.getMessage();
            perror(lasterror);
            return null;
        }
    }
    /**
     *Carica un Synset nuovo. Fallisce in caso di mword gia` presenti (ma
     * non in caso di aggiunta di mword diverse)
     *
     *@param S Synset da inserire
     *@throws Exception In caso di problemi al DB spara un'eccezione
     *@return null, in caso di errore
     */
    public Integer loadSynset(Synset S) throws Exception {
        String TopicURI = S.getConcept();
        String lang = S.getLanguage();
        MWord mword[]=S.getMWords();
        //search the key of the synset, otherwise create a new one
        Integer idwssynset = selectSynset(TopicURI,lang);
        if (idwssynset == null) {
            if ((idwssynset=insertSynset(TopicURI,lang)) == null)
                return null;
        }
        // inserting new mwords
        if ( insertMWord(mword) == null)
            return null;
        con.setAutoCommit(false);
        
        ResultSet rs;
        PreparedStatement sqlsel = con.prepareStatement
        ("INSERT INTO wsoccour (idwssynset, occourence,weight,idwsmword)"+
        " VALUES(?,?,?,(select idwsmword from wsmword where content=?))");
        
        for (int i=0;i<mword.length;++i) {
            try {
                sqlsel.setInt(1,idwssynset.intValue());
                sqlsel.setInt(2,mword[i].getFrequency());
                sqlsel.setDouble(3,mword[i].getWeight());
                sqlsel.setString(4,mword[i].getMWord());
                sqlsel.executeUpdate();
            }
            catch (SQLException sqle) {
                con.rollback();
                con.setAutoCommit(true);
                throw sqle;
            }
        }
        con.commit();
        //con.setAutoCommit(true);
        return idwssynset;
    };
    /**
     * Updates a Synset.
     * @return null for errors, otherwise
     * @param addMode false new values will overwrite older ones, true the new occouurence wil be added to the old one (weight will be overwritten anyway)
     * @param S Synset to insert/update
     * @throws Exception In error
     */
    public Integer updateSynset(Synset S, boolean addMode) throws Exception {
        String TopicURI = S.getConcept();
        String lang = S.getLanguage();
        MWord mword[]=S.getMWords();
        //check the key of the synset, otherwise create a new one
        Integer idwssynset = selectSynset(TopicURI,lang);
        if (idwssynset == null) {
            if (insertSynset(TopicURI,lang) == null)
                return null;
        }
        // insert new mwords
        if ( insertMWord(mword) == null)
            return null;
        con.setAutoCommit(false);
        
        ResultSet rs;
        PreparedStatement sqlsel = con.prepareStatement
        ("SELECT o.occourence,o.idwsmword"+
        " FROM wsoccour o, wsmword w" +
        " WHERE o.idwssynset = ? "+
        " AND o.idwsmword=w.idwsmword" +
        " AND w.content=? FOR UPDATE");
        
        PreparedStatement sqlupd = con.prepareStatement
        ("UPDATE wsoccour SET occourence = ?" +
        " WHERE idwssynset = ? "+
        " AND idwsmword = ?");
        
        PreparedStatement sqlins = con.prepareStatement
        ("INSERT INTO wsoccour (idwssynset, occourence,weight,idwsmword)"+
        " VALUES(?,?,?,(select idwsmword from wsmword where content=?))");
        
        int occ,idwsmword;
        
        for (int i=0;i<mword.length;++i) {
            try {
                sqlsel.setInt(1,idwssynset.intValue());
                sqlsel.setString(2,mword[i].getMWord());
                ResultSet r = sqlsel.executeQuery();
                if (r.next()==false) {
                    sqlins.setInt(1,idwssynset.intValue());
                    sqlins.setInt(2,mword[i].getFrequency());
                    sqlins.setDouble(3,mword[i].getWeight());
                    sqlins.setString(4,mword[i].getMWord());
                    sqlins.executeUpdate();
                }
                else {
                    occ = addMode ? r.getInt(1) : 0;
                    idwsmword = r.getInt(2);
                    
                    //debug
                    System.out.println("--->>updating synset with value: "+(occ + mword[i].getFrequency()));
                    
                    sqlupd.setInt(1,occ + mword[i].getFrequency());
                    sqlupd.setInt(2,idwssynset.intValue());
                    sqlupd.setInt(3,idwsmword);
                    if (sqlupd.executeUpdate() !=1) {
                        con.rollback();
                        con.setAutoCommit(true);
                        throw new Exception("updateSynset: too many rows updated!");
                    }
                    if (r.next()==true) {
                        con.rollback();
                        con.setAutoCommit(true);
                        throw new Exception("updateSynset: too many rows selected!");
                    }
                }
                
                
            }
            catch (SQLException sqle) {
                con.rollback();
                con.setAutoCommit(true);
                throw sqle;
            }
        }
        con.commit();
        //con.setAutoCommit(true);
        return idwssynset;
    };
    /**
     * Get a Sysnset given URI and language
     * @param TopicURI the synset's topic
     * @param lang the synset's language
     * @throws Exception in error
     * @return the Synset requested
     */
    public Synset getSynset(String TopicURI,String lang) throws Exception {
        
        Vector V = new Vector();
        ResultSet rs;
        PreparedStatement sqlprep;
        int i = 0;
        
        String sql="SELECT m.content,o.occourence,o.weight,s.content "+
        " FROM wsmword m, wsoccour o, wssynset s, wstopic w "+
        " WHERE w.idwstopic = s.idwstopic "+
        " AND   o.idwssynset = s.idwssynset "+
        " AND   o.idwsmword = m.idwsmword "+
        " AND   w.tname = ? "+
        " AND   s.lang = ? ";
        
        sqlprep = con.prepareStatement(sql);
        sqlprep.setString(1,TopicURI);
        sqlprep.setString(2,lang);
        rs = sqlprep.executeQuery();
        
        
        while (rs.next()) {
            V.add(new MWord(rs.getString(1),rs.getInt(2),rs.getDouble(3)));
            i++;
        }
        if (i==0)
            return null;
        /* convert a vector in an array */
        V.trimToSize();
        MWord[] tmpmword = new MWord[V.size()];
        for(int j=0; j<V.size(); j++)
            tmpmword[j]=(MWord)V.elementAt(j);
        return new Synset(TopicURI,tmpmword,lang);
        
    }
    /**
     * Get all synset for a giveb language
     * this is not optimized: will call multiple times getSynset
     *
     * @param lang the language requested
     * @throws Exception in error
     * @return an array with all the synset for the given language
     */
    public Synset[] getAnySynset(String lang) throws Exception {
        
        Vector V = new Vector();
        ResultSet rs;
        
        int i=0;
        rs = stmt.executeQuery("select tname from wstopic");
        while (rs.next()) {
            Synset syn = getSynset(rs.getString(1),lang);
            if(syn != null)
                V.add(syn);
            i++;
        }
        if (i==0)
            return null;
        V.trimToSize();
        Synset[] tmpsynset = new Synset[V.size()];
        for(int j=0; j<V.size(); j++)
            tmpsynset[j]=(Synset)V.elementAt(j);
        return tmpsynset;
    }
    /**
     * returns the number of annotation (unexpanded) for a given topic
     * @return the number of annotstion for the given topicuri
     * @param topicURI the topicuri to search
     * @throws Exception in error
     */
    public int countAnnotations(String topicURI)
    throws Exception {
        
        PreparedStatement sqlprep;
        int i;
        int j;
        
        String sql = "SELECT COUNT(a.*) "+
        " FROM wsannotations a, wstopic t "+
        " WHERE a.idwstopic=t.idwstopic "+
        " AND t.tname = ?";
        sqlprep = con.prepareStatement(sql);
        sqlprep.setString(1,topicURI);
        ResultSet rs = sqlprep.executeQuery();
        if ( !rs.next() )
            return -EAGAIN;
        return (rs.getInt(1));
    }
    /**
     * gives back the total number of annotations stored in the database
     * @throws Exception in error
     * @return the total number of annotation
     */
    public int countAnnotations() throws Exception {
        ResultSet rs;
        rs = stmt.executeQuery("select count(*) from wsannotations");
        if (!rs.next())
            return EAGAIN; /*if the recordset is null, something bad
             *happened */
        return rs.getInt(1);
    }
    /**
     * delete an author from the database given the name (and ALL the annotations
     * synset that use it)
     * @param author the author name to delete
     * @throws Exception see deleteGeneric
     * @return see deleteGeneric 
     */    
    public Integer deleteAuthor(String author) throws Exception {
        String v[] = {"name"};
        String w[] = {author};
        return deleteGenericValue("wsauthor",v,w);
    }
    /**
     * delete an author from the database given the key (and ALL the annotations
     * synset that use it)
     * @param idauthor the author to delete
     * @return null in error else the number of topic deleted (1)
     * @throws Exception in error
     */
    public Integer deleteAuthor(Integer idauthor) throws Exception {
        return deleteGenericKey("wsauthor","idwsauthor",idauthor);
    }
    /**
     * delete an annotation type from the database given the key (and ALL the annotations
     * synset that use it)
     * @return null in error else the number of annotations deleted (1)
     * @param idtype the annotation type to delete
     * @throws Exception in error
     */
    public Integer deleteType(Integer idtype) throws Exception {
        return deleteGenericKey("wsanntype","idwsanntype",idtype);
    }
    /**
     * delete an annotation type from the database given the name (and ALL the annotations
     * synset that use it)
     * @param anntype the annotation type to delete
     * @throws Exception see deleteGeneric
     * @return see deleteGeneric
     */    
    public Integer deleteType(String anntype) throws Exception {
        
        String v[] = {"tname"};
        String w[] = {anntype};
        return deleteGenericValue("wsanntype",v,w);
    }
    /**
     * delete a topic, given the topic key
     * @param idtopic the topic to delete
     * @throws Exception see deleteGeneric
     * @return see deleteGeneric
     */    
    public Integer deleteTopic(Integer idtopic)
    throws Exception {
        return deleteGenericKey("wstopic","idwstopic",idtopic);
    }
    /**
     * delete a topic from the database given the URI (and ALL the annotations
     * synset that use it)
     * @param topic the topic to delete
     * @return null in error else the number of topic deleted (1)
     * @throws Exception in error
     */
    public Integer deleteTopic(String topic)
    throws Exception {
        String v[] = {"tname"};
        String w[] = {topic};
        return deleteGenericValue("wstopic",v,w);
    }
    /**
     *Permette di eseguire una select su una generica tabella ed ottenere
     *l'indice corrispondente ad una chiave
     *@param table tabella sa selezionare
     *@param keyColumn Colonna indice della tabella
     *@return indice trovato, altrimenti null
     *@param column Colonna in cui sono presenti i valori da cetrcare
     *@param value Valore del dato da cercare (nella colonna column)
     *@throws Exception In caso di errore spara una eccezione
     */
    private Integer selectGeneric(String table, String keyColumn, String column[], String value[])
    throws Exception {
        PreparedStatement sqlprep;
        Integer i;
        int j;
        String virgola="   ";
        String sql = "SELECT " + keyColumn + " FROM " + table + " WHERE ";
        for (j=0; j<column.length  ;j++) {
            sql = sql + virgola + column[j]+ " = ?";
            virgola = " AND ";
        }
        if (DEBUG)
            System.out.println("SelectGenericSQL sql="+sql);
        sqlprep = con.prepareStatement(sql);
        for (j=0; j<column.length  ;j++) {
            sqlprep.setString(j+1, value[j]);
        }
        ResultSet rs = sqlprep.executeQuery();
        if ( rs.next() ) {
            i = new Integer(rs.getInt(1));
            return i;
        }
        else {
            return null;
        }
    }
    /**
     * Permette di effettuare una insert su una generica tabella
     * @param table la tabella utilizzata
     * @param keyColumn colonna chiave
     * @param seq sequenza da cui prendere i valori
     * @param column colonne da inserire
     * @param value valori da inserire
     * @throws Exception in caso di errore SQL...
     * @return la chiave della riga inserita, null in caso di problemi
     */
    private Integer insertGeneric(String table, String keyColumn, String seq, String[] column, String[] value)
    throws Exception {
        String sql = "";
        Statement st = con.createStatement();
        try {
            int i,j,keyValue;
            sql = "select nextval('"+seq+"')";
            ResultSet r=st.executeQuery(sql);
            if (r == null) {
                return null;
            }
            if (r.next()) {
                keyValue = r.getInt(1);
            }
            else {
                return null;
            }
            sql = "INSERT INTO " + table + "("+ keyColumn;
            for (j=0; j<column.length  ;j++) {
                sql = sql + "," + column[j];
            }
            sql = sql + ") VALUES ("+keyValue;
            for (j=0; j<column.length  ;j++) {
                sql = sql + ",'"+value[j]+"'";
            }
            sql = sql + ")";
            i = st.executeUpdate(sql);
            if (i !=1 ) {
                con.rollback();
                con.setAutoCommit(true);
                return null;
            }
            con.commit();
            //con.setAutoCommit(true);
            return new Integer(keyValue);
        }
        catch (SQLException S) {
            lasterror = "insertGeneric:"+S.getMessage()+"(sql:"+sql+")";
            perror(lasterror);
            return null;
        }
    }
    /**
     *permette di eleiminare una riga da una tabella dato il valore di un campo
     *@param table nome della tabella su cui lavorare
     *@param column colonna del campo
     *@param value valore del campo
     *@throws Exception In caso di problemi al DB spara un'eccezione
     *@return null, in caso di errore, altrimenti il numero di righe eliminate
     */
    private Integer deleteGenericValue(String table, String column[], String value[])
    throws Exception {
        try {
            PreparedStatement sqlprep;
            Integer i;
            int j;
            String virgola ="   ";
            String sql = "DELETE FROM " + table + " WHERE " ;
            for (j=0; j<column.length  ;j++) {
                sql += virgola + column[j]+ " = ?";
                virgola = " AND ";
            }
            sqlprep = con.prepareStatement(sql);
            for (j=0; j<column.length  ;j++) {
                sqlprep.setString(j+1, value[j]);
            }
            sqlprep = con.prepareStatement(sql);
            for (j=0; j<column.length  ;j++) {
                sqlprep.setString(j+1,value[j]);
            }
            i = new Integer(sqlprep.executeUpdate());
            return i;
        }
        catch (SQLException S) {
            lasterror = S.getMessage();
            perror(lasterror);
            return null;
        }
    }
    /**
     * permette di eliminare una riga dalla tabella data la chiave
     * @param table tabella su cui si opera
     * @param keyColumn colonna che contiene la chiave
     * @param value valore della chiave
     * @throws Exception In caso di problemi al DB spara un'eccezione
     * @return null, in caso di errore, altrimenti il numero di righe eliminate
     */
    private Integer deleteGenericKey(String table, String keyColumn, Integer value)
    throws Exception {
        try {
            PreparedStatement sqlprep;
            Integer i;
            String sql = "DELETE FROM " + table + " WHERE " + keyColumn + "= ? " ;
            sqlprep = con.prepareStatement(sql);
            sqlprep.setInt(1,value.intValue());
            i = new Integer(sqlprep.executeUpdate());
            return i;
        }
        catch (SQLException S) {
            lasterror = S.getMessage();
            perror(lasterror);
            return null;
        }
    }
    /**
     *
     * For being able to implement the political of self-management
     * of the conceptual cover of annotations it is necessary to
     * introduce a method getTopicsCoverage.
     * @returns TopicCoverage the result must be an array of couples "topicURI"/"frequecy",
     * where the frequency represents the number of existing annotations
     * reported to a data topic, ordered in increasing way (before the smaller
     * and then the greater ones frequencies...).
     * "null" in wrogn parameter case (ie.
     * @throws Exception in error case
     * @return the object requested
     */
    public TopicCoverage getTopicsCoverage() throws Exception {
        /* ok con PostgreSQL || Oracle */
        String sql = "SELECT w.tname, COUNT(a.*)" +
        " FROM wstopic w LEFT OUTER JOIN wsviewannotations a" +
        " ON w.idwstopic = a.idwstopic" +
        " GROUP BY 1 ORDER BY 2 ASC ";
        Vector V = null;
        TopicCoverage T = null;
        ResultSet rs;
        try {
            rs = stmt.executeQuery(sql);
        }
        catch (Exception SQLe) {
            lasterror = SQLe.getMessage() +" sql:"+sql+".";
            perror(lasterror);
            return null;
        }
        /* now we'll build a temporary Vector to make a TopicCoverage */
        try  {
            V = new Vector();
            int i = 0;
            while (rs.next()) {
                /* we now check if the fragment really exists */
                i++;
                V.addElement(new TopicFrequency(rs.getString(1),rs.getInt(2)));
            };
            if (i == 0)
                return null;
            T = new TopicCoverage(V);
            return T;
        }
        catch (Exception E) {
            lasterror = E.getMessage();
            perror(lasterror);
            return null;
        }
    }
} /* Class DBWrapperImpl ends */
