package it.polito.elite.doseWS.wrappers;

import java.util.*;

public interface Ontology {
	
	/**
	 * Provides a Vector containing all ontology concepts as URIs.
	 * @return The Vector containing all ontology concepts.
	 */
	public Vector getAllClasses();
	
	
	public Set getRoots();
	
	/** Retrieves all direct sub-classes of the given concept.
	 * @param uri The URI of the &quot;father&quot; concept.
	 * @return An Iterator containing all sub-classes expressed as URIs.
	 */
	public Iterator getSubClasses(String topicURI);
	
	/** Retrieves all direct super-classes of the given concept.
	 * @param uri The URI of the concept.
	 * @return An Iterator containing all super-classes expressed as URIs.
	 */
	public Iterator getSuperClasses(String topicURI);
	
	
	public Hashtable getAllRelations();
	
	
	public Vector getRelationEnds(String topicURI, String relationURI);
	
	/**
	 * Returns the language-dependent title of a concept.
	 * @param topicURI The URI of a concept.
	 * @param language The language code.
	 * @return The title.
	 */
	public String getTitle(String topicURI, String language);
	
	/**
	 * Returns the language-dependent definition of a concept.
	 * @param topicURI The URI of a concept.
	 * @param language The language code.
	 * @return The definition.
	 */	
	public String getDefinition(String topicURI, String language);
}